/************************************************************************
*	File Name:	rf.c
*
*	Function:	This drivers includes some functions for the Bluetooth radio
*				tranciever used in the system.
*
************************************************************************/

#include "config.h"
#include "chipdef.h"
#include "drivers.h"

#if (RF_CHIP == LMX5250)
#define SETUP_LENGTH 54
#endif

#if (RF_CHIP == LMX5251)
#define SETUP_LENGTH 40
#endif

#if (RF_CHIP == LMX5252)
#define SETUP_LENGTH 22
#endif


#ifdef RF_CHIP

#define BIT_SEQ_SIZE 25
#define CCB_HEAD_READ  0x0B
#define CCB_HEAD_WRITE 0x0A

static const pindesc_t pindesc =
{
	BTLLC_MUX_PORT,
	CCB_SCLK, 
	CCB_SDAT,
	CCB_SLEN
};

struct memory
{
	unsigned char addr;
	unsigned short data;
};

const struct memory set12mhz[] =
{


#if (RF_CHIP == LMX5250)
  {0x00,  0x0100},
  {0x00,  0x0100},  // Repeat the first instruction to fix start-up problem
  {0x11,  0x0000},
  {0x12,  0x030C},
  {0x15,  0x0000},
  {0x16,  0x0306},
  //page 0 return
  {0x00,  0x0000},
  {0x01,  0x0000},
  //{0x09,  0x030E},
  {0x0b,  0x0060},
  {0x19,  0x8483},
  {0x07,  0x0006},
  //page 1 init
  {0x00,  0x0100},  // Previously in RF driver
  {0x03,  0x00A4},
  {0x06,  0x4348},
  {0x07,  0x321A},
  {0x08,  0x1000},
  {0x09,  0x0010},
  {0x0C,  0x6806},
  {0x0E,  0x0000},
  {0x0F,  0x0000},
  {0x10,  0xFFF1},
  {0x11,  0x0000},
  {0x12,  0x030C},
  {0x15,  0x0000},
  {0x16,  0x0306},
  {0x17,  0x06DC}, 
  {0x18,  0xC7FC},
  {0x19,  0x2F24}, //TM0408
  {0x1A,  0x6924},
    //page 2 init     
  {0x00,  0x0200},
  {0x02,  0x1806},
  {0x05,  0x0004},
  {0x06,  0x0200},
  {0x07,  0x0300},
  {0x08,  0x0700},
  {0x09,  0x0700},
  {0x0A,  0x1E50},
  {0x0C,  0x3C1E},
  {0x0D,  0x3228},
  {0x0E,  0x0922},
  //page 0 return
  {0x00,  0x0000},
  {0x01,  0x0000},
  {0x02,  0x6400},
 // {0x03,  0x6100},   //TM0308
  {0x03,  0x6500},   //TM0308    Timing for PA on
  {0x04,  0x4700},
  {0x05,  0xA400},
  {0x08,  0x3073},
  //{0x09,  0x020E},
  {0x0b,  0x006C},
  {0x10,  0x1A01},
  {0x19,  0x8483},
  {0x1C,  0x1F77},
  {0x07,  0x0006},
  {0x09,  0x0360},
  {0x00,  0x0000}
#endif

#if (RF_CHIP == LMX5251)
// Address,  Value
 {0x0B, 0x0000},  //   0          
 {0x0B, 0x0000},  //   1          
 {0x00, 0x0100},  //   2          
 {0x11, 0xFFDF},  //   3         
 {0x12, 0x230B},  //   4         
 {0x03, 0x02B3},  //   5         
 {0x06, 0x4650},  //   6         
 {0x07, 0x3228},  //   7         
 {0x08, 0x0600},  //   8         
 {0x09, 0x0007},  //   9         
 {0x0C, 0x5094},  //  10         
 {0x0E, 0x9000},  //  11         
 {0x0F, 0xA994},  //  12         
 {0x10, 0x566A},  //  13         
 {0x13, 0x8005},  //  14         
 {0x16, 0x9200},  //  15         
 {0x17, 0x28DC},  //  16         
 {0x18, 0x36DB},  //  17         
 {0x19, 0x58E3},  //  18         
 {0x00, 0x0300},  //  19         
 {0x0F, 0x004B},  //  20         
 {0x0C, 0x2030},  //  21         
 {0x0D, 0x14C0},  //  22         
 {0x14, 0x002F},  //  23         
 {0x00, 0x0000},  //  24         
 {0x12, 0x007E},  //  25         
 {0x05, 0xA400},  //  26         
 {0x05, 0x0400},  //  27         
 {0x02, 0x6EB4},  //  28         
 {0x03, 0x5C00},  //  29         
 {0x04, 0x5B04},  //  30         
 {0x0D, 0x0064},  //  31
 {0x0C, 0x0220},  //  32         
 {0x10, 0x1F00},  //  33         
 {0x09, 0x066E},  //  34         
 {0x00, 0x0300},  //  35
 {0x10, 0x1845},  //  36
 {0x00, 0x0100},  //  37         
 {0x1A, 0x36A4},  //  38         
 {0x00, 0x0000}   //  39
#endif

#if (RF_CHIP == LMX5252)
    {  17,  0x0808},
    {  16,  0x0000},
    {  15,  0x074A},
    {  15,  0xA100},
    {  14,  0x0000},
    {  14,  0x2FB7},
    {  12,  0x0000},
    {  11,  0x0000},
    {  11,  0x0000},
    {  10,  0xFFFF},
    {  10,  0xF801},
    {   9,  0x0012},
    {   9,  0x4F80},
    {   8,  0x0002},
    {   7,  0x001F},
    {   6,  0x0080},
    {   6,  0x0040},
    {   5,  0x08C0},
    {   4,  0x001C},
    {   4,  0xD000},
    {   2,  0x0028},
    {   1,  0x6405}
#endif
};


/******************************************************************************  
    Function: rf_write_reg

	Description
	-----------
    This function writes a value to a given RF chip register.

	Arguments
	---------
	addr:
	The RF chip register address.

	data:
	A 16-bit word containing the value to be written to the register.

	Retrurns:
*******************************************************************************/

void rf_write_reg(unsigned char addr, unsigned short data)
{
	unsigned char bitseq[4];
	

	/* Header plus 4 MS bits of address */
	bitseq[0] = (CCB_HEAD_WRITE << 4) | ((addr >> 1) & 0xf);

	/* LS bit of address plus 7 MS bits of data (bits 15..9) */
	bitseq[1] = (addr << 7) | (unsigned char) ((data >> 9) & 0x7f);

	/* Bits 8..1 of the data. */
	bitseq[2] = (unsigned char) (data >> 1);

	/* LS bit of data (bit 0) */
	bitseq[3] = (unsigned char) (data << 7); 

	/* Now bit bang the bit sequence. */
	sibb_write((pindesc_t *)&pindesc, bitseq, BIT_SEQ_SIZE);

}

/******************************************************************************  
    Function: rf_set_bbclk

	Description
	-----------
    This function sets the RF chip to generate a 12MHz baseband clock.

	Arguments
	---------
*******************************************************************************/
void rf_set_bbclk(void)
{
	unsigned int step;

	for (step = 0; step < SETUP_LENGTH; step++)	// write the structure data
		rf_write_reg(set12mhz[step].addr, set12mhz[step].data);
}

#endif
