/************************************************************************
*	File Name:	switch.c
*
*	Function:	This is a generic switch driver. It provides read services
*				from any type of hardware switch that can be used by
*				to determine the current setting of a certain parameter. 
*
************************************************************************/ 

#include "crcommon.h"
#include "config.h"
#include "boarddef.h"


/******************************************************************************  
    Function: switch_on

	Description
	-----------
    This function reads the state of a specified switch.

	Arguments
	---------
	switch_num:
	The logical number of the switch on the board
	Note: on each board we assign logical switch numbers. These don't
	necessarily match the physical switch number so we provide the
	physical to logical mapping scheme for each board. The reason
	for using logical switch numbers is to allow an application to use
	one logical switch number across all boards.

	Returns:
	--------
	A boolean value indicated the switch state. TRUE = ON, FALSE = OFF.
*******************************************************************************/
boolean switch_on(unsigned int switch_num)
{
#if (NUM_SWITCHES != 0) 
/* Save redundant definitions if board has 0 switches. Execute the following
   code only if we have switches on the board.
*/
	if (switch_num >= NUM_SWITCHES)
		return FALSE;

	return (((SWITCHREG >> (SWITCH0_OFFSET + switch_num)) & 0x1) ^
			(!SWITCH_POLARITY));
#else
	return FALSE;
#endif

}


/******************************************************************************  
    Function: switch_count

	Description
	-----------
    This function returns the number of switches on the board.

	Arguments
	---------
	None.

	Returns:
	--------
	The number of of switches on the board.
*******************************************************************************/

unsigned int switch_count(void)
{
	return NUM_SWITCHES;
}


