/******************************************************************************
*  File:	biu.c
*  
*  This is the Bus Interface Unit (BIU) driver.
******************************************************************************/

#include "chipdef.h"

biu_t * const biu = (biu_t *)BIU_BASE_ADDR;

/******************************************************************************  
    Function: biu_config_zone

	Description
	-----------
    This function configures a given memory or I/O zone according to specified
    parameters.

	Arguments
	---------
	zone:
	The zone id. One of ZONE0, ZONE1, ZONE2, IOZONE.

	ws:
	Number of wait states to be applied to this zone.

	hs:
	Number of hold states to be applied to this zone.

	fr:
	A fast-read flag. If TRUE fast-read mode will be enabled for the zone.
	Ignored for the I/O zone.

*******************************************************************************/
void biu_config_zone(unsigned int zone, unsigned int ws, unsigned int hs,
                     boolean fr)
{
	unsigned short mask;

	mask = ws | (hs << 3);
	if (fr && zone != IOZONE)
		mask |= 0x0800;

	switch (zone) {
    case ZONE0:
	  SZCFG0 = (SZCFG0 & 0xf7e0) | mask; 
	  break;
	case ZONE1:
	  SZCFG1 = (SZCFG1 & 0xf7e0) | mask;
	  break;
    case ZONE2:
	  SZCFG2 = (SZCFG2 & 0xf7e0) | mask;
	  break;
	case IOZONE:
	  IOCFG = (IOCFG & 0xffe0) | mask;
	  break;
  }
}

/******************************************************************************  
    Function: biu_set_early_write

	Description
	-----------
    This function sets the early-write mode of the BIU.

	Arguments
	---------
	er:
	Boolean flag to indicate the early-write mode. TRUE - early-write is
	on. FALSE - early-write is off (late-write is on).

*******************************************************************************/
void biu_set_early_write(boolean er)
{
	if (er)
		BCFG |= EWR;
	else
		BCFG &= (~EWR);
}
					 
