/******************************************************************************
*  File:	icu.c
*  
*  This is the Interrupt Control Unit (ICU) driver.
******************************************************************************/

#include "chipdef.h"

icu_t * const icu = (icu_t *)ICU_BASE_ADDR;

/******************************************************************************  
    Function: icu_init

	Description
	-----------
    This function initializes the ICU hardware. It mask all interrupts.

	Arguments
	---------
	None.

*******************************************************************************/
void icu_init(void)
{
  /* Mask all interrupts */
  IENAM0 = 0;
#ifdef IENAM1 
  IENAM1 = 0;
#endif
#ifdef IENAM2 
  IENAM2 = 0;
#endif

}

/******************************************************************************  
    Function: icu_unmask

	Description
	-----------
    This function unmasks a single interrupt specified by its IRQ number.

	Arguments
	---------
	irq:
	The IRQ number of the interrupt to be unmasked.

*******************************************************************************/
void icu_unmask(unsigned int irq)
{
  if (irq < 16)
	/* If IRQ is between 0-15 the mask bit is in IENAM0 */
    IENAM0 |= (1 << irq);
#ifdef IENAM1
  else if (irq < 32)
	/* If IRQ is between 16-31 the mask bit is in IENAM1 */
    IENAM1 |= (1 << (irq - 16));
#endif
#ifdef IENAM2
  else if (irq < 48)
	/* If IRQ is between 32-47 the mask bit is in IENAM2 */
    IENAM2 |= (1 << (irq - 32));
#endif
}

/******************************************************************************  
    Function: icu_mask

	Description
	-----------
    This function masks a single interrupt specified by its IRQ number.

	Arguments
	---------
	irq:
	The IRQ number of the interrupt to be masked.

*******************************************************************************/
void icu_mask (unsigned int irq)
{
  if (irq < 16)
	/* If IRQ is between 0-15 the mask bit is in IENAM0 */
    IENAM0 &= ~(1 << irq);
#ifdef IENAM1
  else if (irq < 32)
	/* If IRQ is between 16-31 the mask bit is in IENAM1 */
    IENAM1 &= ~(1 << (irq - 16));
#endif
#ifdef IENAM2
  else if (irq < 48)
	/* If IRQ is between 32-47 the mask bit is in IENAM2 */
    IENAM2 &= ~(1 << (irq - 32));
#endif

}
