/******************************************************************************
*	File:	intable.c
*  
*	This file contains the interface for the CP3BT1X/CP3BT2X interrupt 
*	dispatch table.
******************************************************************************/

/* The interrupt dispatch table is implemented as an array of function to
   pointers. The non-NULL pointers point to interrupt handlers that have
   so far been developed for CP3BT1X/CP3BT2X peripherals.
   In case a specific handler is not used in a particular application
   a stub function will resolve the reference to its function symbols.
   This will make sure that the reference from the interrupt dispatch table
   is resolved even when the handler is not used. The stub functions are
   included in the interrupt library (each one in a separate object).
*/

#include "config.h"
#include "chipdef.h"


void (INTERRUPT * const _dispatch_table[])() = {

#if (CHIP_FAMILY == CP3BT1X) 
#include "cp3bt1xisrdef.h"
#endif

#if (CHIP_FAMILY == CP3BT2X) 
#include "cp3bt2xisrdef.h"
#endif

};

/******************************************************************************  
    Function: null_isr

	Description
	-----------
    This is a default interrupt service routine for all those interrupts
    that are not handled yet. For development purposes we generate a
	breakpoint trap in these cases to catch stray interrupts.

	Arguments
	---------
	None.
	
	Return value
	------------
	None.
*******************************************************************************/
#ifdef __NSC_CRCC__
#pragma interrupt(null_isr)
void null_isr(void)
{
	_excp_(bpt);
}
#endif

