/************************************************************************
	File Name : $File$
	$Date$
	$Revision$

	Function:	This is the driver for the Random Number Generator
		peripheral. 
			
************************************************************************/


#include "drivers.h"

#include <stdio.h>
#include <string.h>
#include "crcommon.h"
#include "chipdef.h"
#include "boarddef.h"

#ifdef UCOS
#include "ucos.h"
#endif

#ifdef RNG_BASE_ADDR

static rng_t * const rng = (rng_t *)RNG_BASE_ADDR;

static void (*rng_cbk_func)() = NULL;

/******************************************************************************  
    Function: rng_init

	Description
	-----------
    This function initialises the RNG

	Arguments
	---------

*******************************************************************************/

void rng_init( uint32 clk_ratio )
{
	RNGDIVL = clk_ratio;
	RNGDIVH = (clk_ratio >> 16) & RNGDIV1716;

	// Don't want any interrupts just yet
	RNGCST &= ~RNG_IMASK;

	icu_unmask(RNG_IRQ); /* Unmask Interrupt controller line */

	RNGCST |= RNGE; // Enable Block
}

/******************************************************************************  
    Function: rng_init

	Description
	-----------
    This function initialises the RNG

	Arguments
	---------

*******************************************************************************/

uint rng_get( void rng_cbk( uint rand ) )
{
	if (rng_cbk) // Do one time callback
	{
		rng_cbk_func = rng_cbk;

		// Setup interrupt
		RNGCST |= RNG_IMASK;

		return 0;
	}
	else // Block until value ready then return it
	{
		while(!(RNGCST & DVALID)); 

		return RNGD;
	}
}

/******************************************************************************  
    Function: rng_shutdown

	Description
	-----------
    This function initialises the RNG

	Arguments
	---------

*******************************************************************************/
void rng_shutdown( void )
{
	icu_mask(RNG_IRQ); /* Mask Interrupt controller line */

	RNGCST &= ~RNGE; // Disable Block
}

/******************************************************************************  
    Function: rng_isr

	Description
	-----------
    This function is the RNG interrupt handler. 

	Arguments
	---------
	None.
	
	Return value
	------------
	None.
*******************************************************************************/

#ifdef __NSC_CRCC__
#pragma interrupt(rng_isr)
void rng_isr(void)
#endif
#ifdef __IAR_SYSTEMS_ICC__
#ifdef __ICCCR16C__
#pragma vector=(16+RNG_IRQ)
__interrupt void rng_isr(void)
#else
__interrupt[(16+RNG_IRQ)*IDT_WIDTH] void rng_isr(void)
#endif
#endif
{	
	if (rng_cbk_func)
	{
		// Disable interrupts - one time callback
		RNGCST &= ~RNG_IMASK;

		rng_cbk_func( RNGD );
	}
}

#endif
