/******************************************************************************
*  File:	traps.c
*  
*  This file contains useful trap handlers.
******************************************************************************/

#include "chipdef.h"

#define UND_TRAP_VECTOR 10
#define IAD_TRAP_VECTOR	12


/******************************************************************************  
    Function: und_isr

	Description
	-----------
    This is the Undefined Instruction (UND) trap service routine. For development
    purposes we generate a breakpoint trap in these cases  to stop execution
    immediately.

	Arguments
	---------
	None.
	
	Return value
	------------
	None.
*******************************************************************************/
#ifdef __NSC_CRCC__
#pragma interrupt (und_isr)
void und_isr(void)
{
	_excp_(bpt);
}
#endif
#ifdef __IAR_SYSTEMS_ICC__
#ifdef __ICCCR16C__
#pragma vector=(UND_TRAP_VECTOR)
__interrupt void und_isr(void)
#else
__interrupt[(UND_TRAP_VECTOR)*IDT_WIDTH] void und_isr(void)
#endif
{
	__raise_exception(__BPT);
}
#endif

/******************************************************************************  
    Function: iad_isr

	Description
	-----------
    This is the Ilegal Address (IAD) trap service routine. For development
    purposes we generate a breakpoint trap in these cases  to stop execution
    immediately.

	Arguments
	---------
	None.
	
	Return value
	------------
	None.
*******************************************************************************/

#ifdef __NSC_CRCC__
#pragma interrupt (iad_isr)
void iad_isr(void)
{
	_excp_(bpt);
}
#endif

#ifdef __IAR_SYSTEMS_ICC__
#ifdef __ICCCR16C__
#pragma vector=(IAD_TRAP_VECTOR)
__interrupt void iad_isr(void)
#else
__interrupt[(IAD_TRAP_VECTOR)*IDT_WIDTH] void iad_isr(void)
#endif
{
	__raise_exception(__BPT);
}
#endif

