/******************************************************************************
*  File:	AudioDemo.c
*  
*  Performs the audio demos for Recording and Playback.
******************************************************************************/

#include "drivers.h"
#include "boarddrv.h"
#include "dbgmsg.h"

#include "AudioDemo.h"

#define BUF_SIZE			64
#define AUDIO_BLOCK_SIZE	0x20000
#define AUDIO_BLOCK_START	0x400000	

static dl_t mem2audio_dl_s, audio2mem_dl_s;
static dl_t *mem2audio_dl, *audio2mem_dl;

static unsigned char mem2audio_buf[BUF_SIZE]; 
static unsigned char audio2mem_buf[BUF_SIZE];

static unsigned char *mem2audio_ptr, *audio2mem_ptr;
static unsigned char FAR *voicemsg_ptr;
static unsigned long total_samples, voicemsg_size;

unsigned int volume;

/******************************************************************************  
*   Function: AudioInitialize
*
*	Description
*	-----------
*   This function initializes the audio to be used with the audio demo 
*	in this file. 
*  
*******************************************************************************/
void AudioInitialize (void)
{
	mem2audio_dl = &mem2audio_dl_s;
	audio2mem_dl = &audio2mem_dl_s;
  
	voicemsg_ptr = (unsigned char FAR *) AUDIO_BLOCK_START;
	total_samples = 0;

	dl_init     (mem2audio_dl, mem2audio_buf, sizeof(mem2audio_buf));
	dl_init     (audio2mem_dl, audio2mem_buf, sizeof(audio2mem_buf));
	dl_init_ptr (mem2audio_dl, mem2audio_ptr);
	dl_init_ptr (audio2mem_dl, audio2mem_ptr);

	codec_init  (mem2audio_dl, audio2mem_dl, NULL, NOCVSDCONV);
}

/******************************************************************************  
*   Function: Record
*
*	Description
*	-----------
*   Records a block of data with size AUDIO_BLOCK_SIZE
*  
*******************************************************************************/
void Record (void)
{
	unsigned int count, i;

	AudioInitialize();

	codec_start();

	while (TRUE)
	{
		count = dl_cnt(audio2mem_dl);

		if ((total_samples + count) > AUDIO_BLOCK_SIZE)
			break;

		for (i = 0; i < count; i++) 
		{
		  *voicemsg_ptr = *audio2mem_ptr;
		  voicemsg_ptr++;
		  dl_advance(audio2mem_dl, audio2mem_ptr);
		}

		total_samples += count;
		dl_c_commit(audio2mem_dl, count);

	}

	codec_stop();
	voicemsg_size = total_samples;
}

/******************************************************************************  
*   Function: Playback
*
*	Description
*	-----------
*   Plays back the block recorded by the Record function
*  
*******************************************************************************/
void Playback (void)
{
	unsigned int count, i;

	AudioInitialize();

	codec_start();

	while (TRUE)
	{
		count = dl_p_space_available(mem2audio_dl);

		if ((total_samples + count) > voicemsg_size)
			break;


		for (i = 0; i < count; i++) 
		{
			*mem2audio_ptr= *voicemsg_ptr;
			voicemsg_ptr++;
			dl_advance(mem2audio_dl, mem2audio_ptr);
		}

		total_samples += count;
		dl_p_commit(mem2audio_dl, count);
	}

	codec_stop();
}









