
/******************************************************************************  
*
*	File Name:	CPDemo.c
*
*	Function:	BDB Demo program
*
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>		  
#include <string.h>
#include <ctype.h>
#include "crcommon.h"
#include "config.h"         
#include "boarddrv.h"
#include "dbgmsg.h"
#include "vt100.h"
#include "heap.h"	
#include "protocol.h"
#include "commintp.h"
#include "drivers.h"

//
//  USB stuff
//
#define N_EPS_USED			  7
#define USB_BUFFER_SIZE		200
#define USE_ASYNCH

COMMBLOCK HostInput;

int	i, BlockIndex, iCount;
static unsigned char   DataBlock[256];
boolean BlinkON;

void usb_evt_cb (usb_evt_t evt, ep_id_t ep);

extern uint8	ucBuffer[N_EPS_USED][USB_BUFFER_SIZE];
extern uint		last_count  [N_EPS_USED];
extern uint		last_rx_ep;
extern boolean  USBConfigured;

void Blink(void);

/******************************************************************************  
*
*   Function: Main
*
*	Description
*	-----------
*		Main function for the CPDemo.
*
*		- Initializes USART and USB
*		- Checks for data on USB and reflects it back to the host.
*		- Gets the data packets and calls ProcessHostCommands to process them
*		  accordingly.
*
*******************************************************************************/
int main(void)
{
	usb_init_info_t usbinitinfo;

	TCRSetClock(_12MHZPLL); 

	//
	// Configure BIU zones. 
	//
	biu_config_zone (ZONE0,  0, 0, TRUE);
	biu_config_zone (ZONE1,  0, 0, TRUE);
	biu_config_zone (ZONE2,  1, 1, TRUE);
	biu_config_zone (IOZONE, 2, 2, FALSE);

	SYSCFGEnableIoExpansion();

	led_7seg_set (0x00);

	//
	// Enable interrupts 
	//
	icu_init();
	set_i_bit();

	//
	// Initialize EEPROM
	//
	eep_init();

	biu_set_early_write(FALSE);	
	timer_init();
  
  //
	// Set the wakeup call to blink the led once per second
  //
	timer_set_wakeup(Blink, 50, TRUE);

	//
	//	Initialize the USART 
	//
	usart_dma_init(BPS_115200, HW_FLOW_CONTROL, NULL);	

  led_init();

	//
	//	Initialize USB
	//
	usbinitinfo.evt_clbk      = usb_evt_cb;
	usbinitinfo.vendor_clbk   = NULL;
	usbinitinfo.class_clbk    = NULL;
	usbinitinfo.set_desc_clbk = NULL;

	usb_init( &usbinitinfo );

  //
  //  Set flag to start blinking LED 0
  //  Clear all LEDs  
  //
  BlinkON = TRUE;
	led_set_all(0x0000);   
   	 
  last_rx_ep = 0;
	HostInput.CommandID = 0x0000;
    
	while (HostInput.CommandID != STOP_COM)
	{                    
		BlockIndex=0;

		//
		//	Wait for a valid Block (EOB in the correct spot)
		//
		do
		{
			//
			//	Wait for any char in the USART
			//
			do
			{
				iCount = usart_dma_rx_count();

				if(USBConfigured)
				{
					usb_rx_data (RXEP1, ucBuffer[RXEP1], USB_BUFFER_SIZE);
					usb_rx_data (RXEP2, ucBuffer[RXEP2], USB_BUFFER_SIZE);
					usb_rx_data (RXEP3, ucBuffer[RXEP3], USB_BUFFER_SIZE);

					USBConfigured=FALSE;
				}

				//
				//	Check if we get USB data
				//
				if (last_rx_ep)
				{
				  for( i=0; i < last_count[last_rx_ep]; i++ )
					ucBuffer[last_rx_ep-1][i] = ucBuffer[last_rx_ep][i];  

				  usb_tx_data ((ep_id_t)(last_rx_ep-1), ucBuffer[last_rx_ep-1], last_count[last_rx_ep]);
				  usb_rx_data ((ep_id_t)    last_rx_ep, ucBuffer[last_rx_ep],          USB_BUFFER_SIZE);
				  last_rx_ep = 0;
				}

			} while (iCount == 0);
      
			usart_dma_rx (&DataBlock[BlockIndex], iCount);
			BlockIndex+=iCount;
			iCount = usart_dma_rx_count();
		} while (DataBlock[BlockIndex-1] != EOB);

		memcpy((char *) &HostInput,  DataBlock, BlockIndex);
       
    BlinkON = FALSE;

		ProcessHostCommands(&HostInput);
	}
  
	return 0;
}

void Blink(void)
{
  //
	// Toggle LED 0.
  //
  if (BlinkON) 
  {
	  led_toggle(0);
 	  led_toggle(1);
  }
}

