#ifndef _descrpt_h_
#define _descrpt_h_


#define BIT7 0x80
#define BIT6 0x40
#define BIT5 0x20

#define MAXNOOFENDPOINTS 6			// Max no of endpoints for CR16BT
									// Change for new devices if neccessary

#define ATTR_RESERVED BIT7
#define ATTR_SELFPOWERED BIT6
#define ATTR_REMOTE_WAKEUP BIT5

#define USB_EP0_MAX_PACKET_SIZE 8

// bDeviceClass
#define USB_DEVICE_CLASS 0xFF

// bDeviceSubClass
#define USB_DEVICE_SUBCLASS 0xFF

// bDeviceProtocol
#define USB_DEVICE_PROTOCOL 0xFF

// Define users USB defined VID here
#define USB_VID 0x0400
// Define users USB PID here
#define USB_PID 0xBAAD      // PID for original code
//#define USB_PID 0xABCE      // PID for new code using latest 5.21 Jungo WinDriver release

// Define USB Version supported here
//#define USB_VER	0x0100			// Rev 1.0
#define USB_VER		0x0110			// Rev 1.1
//#define USB_VER		0x0200		// Rev 2.0

// Define EP addresses and directions
#define TX_EP1			0x81
#define RX_EP2			0x01

#define TX_EP3			0x82
#define RX_EP4			0x02

#define TX_EP5			0x83
#define RX_EP6			0x03


// Ensure that the structures within this header file are packed tightly - for NS toolset
#ifdef __IAR_SYSTEMS_ICC__
#pragma pack(1)
#endif 
#ifdef __NSC_CRCC__
#pragma set_options("-mbiggest-struct-alignment-8")
#endif

//---------------- Standard Feature Selectors  -----------------------
enum {
  USB_DEVICE_HALT,
  USB_REMOTE_WAKEUP };

#define NO_OF_CONFIGURATIONS 1

// Set up Interfaces and Alternate settings for each configuration
// using following format

// This configuration has one interface
const t_EndpointSetup Config1Interface0[1] =
{
	{0x7E, 0x00}		// Uses EP1 to EP6, No ISO support
};

// Now we set up pointers to each of those config/interface arrays
const t_EndpointSetup * Config1[1] =
{
	Config1Interface0
};

// Now we need to point to each of these configurations
const t_EndpointSetup ** EPConfiguration[1] =
{
	Config1
};

//----------------------      Other    ----------------------------
typedef struct {
  uint8  bLength;
  uint8  bDescriptorType;
} UsbDescriptorType;

/* All descriptors are sent concatenated to the host. By statically
   forcing the structures to be stored contiguously in memory by means
   of UsbTotalDescriptorType, we need not do run-time concatenation
   and thus can send the descriptors without using heap and without
   doing memcpy. Assumes 1 byte alignment of structures. */

typedef struct {
  // the general configuration descriptor
  UsbConfigurationDescriptorType Configuration1Descriptor;
  // interface for testing - has 3 endpoints apart form nr. 0,
  // which need not be returned to the host.
  UsbInterfaceDescriptorType interface100;
  UsbEndpointDescriptorType BulkInEndpointDescriptor100;
  UsbEndpointDescriptorType BulkOutEndpointDescriptor100;
  UsbEndpointDescriptorType IntInEndpointDescriptor100;
  UsbEndpointDescriptorType IntOutEndpointDescriptor100;
  UsbEndpointDescriptorType IsoInEndpointDescriptor100;
  UsbEndpointDescriptorType IsoOutEndpointDescriptor100;

	// TODO: User can add extra endpoints and interfaces here onwards
} UsbTotalDescriptorType1;

// Standard Descriptor Types
enum
{
  DEVICE_DES=1,
  CONFIGURATION_DES,
  STRING_DES,
  INTERFACE_DES,
  ENDPOINT_DES
};

// Endpoint types
enum {
  CONTROL_EP,
  ISOCHRONOUS_EP,
  BULK_EP,
  INTERRUPT_EP 
};

// Language and String Indices
enum {
  USB_US_ENGLISH_LANGUAGE_INDEX,
  USB_NROF_LANGUAGES
};

// Enumerates Index for Configurations
enum
{
	CONFIGURATION1,
//	CONFIGURATION2,
	// TODO: Add futher configurations here

	NROF_CONFIG_DESCRIPTORS
};


// Enumerates Index values for Strings
// Add indexes for further configurations and interfaces under those configurations
enum {
  LANGUAGE_ID_INDEX,
  MANUFACTURER_STRING_INDEX,
  PRODUCT_STRING_INDEX,
  SERIAL_NR_STRING_INDEX,
  CONFIGURATION1_STRING_INDEX,
  INTF100_STRING_INDEX,
  NROF_STRING_DESCRIPTORS
};

const uint8 ucNROF_STRING_DESCRIPTORS = NROF_STRING_DESCRIPTORS;

/* String descriptors */
// See USB 2.0 Sec 9.6.7
// wLangID[] should be repeated as per users choice of languages

const uint8 USBLangId [4] = 
{ 
  sizeof(USBLangId),	// bLength - Size of this descriptor in bytes 
  STRING_DES,			// bDescriptorType - String Descriptor is type 0x03
  0x09,					// wLangID[0] as per http://www.usb.org/developers/docs.html (English UK)
  0x04					//
  						// wLangID[1]
						// .
						// .
						// .
						// wLangID[n]
};

// TODO: Note all string descriptors are UNICODE (www.unicode.org) so
//		 any language can be supported.

const uint8 manufacturerstr[]	= "\x2E\x03N\0a\0t\0i\0o\0n\0a\0l\0 \0S\0e\0m\0i\0c\0o\0n\0d\0u\0c\0t\0o\0r\0";
const uint8 productstr[]		= "\x20\x03U\0S\0B\0 \0D\0e\0m\0o\0 \0D\0e\0v\0i\0c\0e\0";
const uint8 serialnrstr[]		= "\x08\x03\x34\0.\0\x39\0";
const uint8 configurationstr1[]	= "\x20\x03\x41\0B\0C\0 \0E\0l\0m\0o\0 \0D\0e\0v\0i\0c\0e\0" ;
const uint8 interface100str1[]	= "\x20\x03X\0Y\0Z\0 \0G\0o\0o\0p\0 \0D\0e\0v\0i\0c\0e\0" ;

const uint8 * const lang1StringDescriptors [NROF_STRING_DESCRIPTORS] =
{
	manufacturerstr,
	productstr,
	serialnrstr,
	configurationstr1,
	interface100str1
};

//*******************  The Device Descriptor ***********************//

const UsbDeviceDescriptorType DeviceDescriptor =
{
  sizeof(UsbDeviceDescriptorType),// bLength
  DEVICE_DES,                 // bDescriptorType
  USB_VER,                          // bcdUSB
  USB_DEVICE_CLASS,        // bDeviceClass
  USB_DEVICE_SUBCLASS,              // bDeviceSubClass 
  USB_DEVICE_PROTOCOL,      // bDeviceProtocol
  USB_EP0_MAX_PACKET_SIZE,        // bMaxPacketSize0
  USB_VID, 
  USB_PID,
  0x0100,                          // bcdDevice 
  MANUFACTURER_STRING_INDEX,  // iManufacturer
//	0x00,
  PRODUCT_STRING_INDEX,       // iProduct
//	0x00,
  SERIAL_NR_STRING_INDEX,     // iSerialNumber
//	0x00,
  NO_OF_CONFIGURATIONS        // bNumConfigurations
};

// TODO: This should perhaps be setup as an array of configurations so that
//		 multiple configurations can be supported, and selected.

const UsbTotalDescriptorType1 Configuration1 =
{
	{
	sizeof(UsbConfigurationDescriptorType), // bLength
	CONFIGURATION_DES,          // bDescriptorType
	sizeof(UsbTotalDescriptorType1), // wTotallength
	// TODO: User chooses number of interfaces here
	1,							// bNumInterfaces
	1,                              // bConfigurationValue
	CONFIGURATION1_STRING_INDEX, // index of string descriptor
	ATTR_SELFPOWERED | 
	ATTR_REMOTE_WAKEUP | 
	ATTR_RESERVED, // bmAttributes
	0x10                              // maxpower in 2mA units
	}, //;
	 
	{
	sizeof (UsbInterfaceDescriptorType), // bLength
	INTERFACE_DES,              // bDescriptorType
	// NOTE: Number of interfaces starts at zero! USB 2.0 Sec 9.6.5
	0,                              // bInterfaceNumber
	0,                              // bAlternateSetting
	6,                              // bNumEndpoints
	0xFF,                           // bInterfaceClass
	0xFF,                           // bInterfaceSubClass
	0xFF,                           // bInterfaceProtocol
	INTF100_STRING_INDEX             // iInterface 
	} //;
	,
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	RX_EP2,                    // bEndpointAddress
	BULK_EP,                       // bmAttributes            
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                  // bInterval (always 0 for bulk)
	}//;
	,
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	TX_EP1,                   // bEndpointAddress
	BULK_EP,                       // bmAttributes
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                  // bInterval (always 0 for bulk)
	}//;
	,
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	RX_EP4,                     // bEndpointAddress
	BULK_EP,                  // bmAttributes            
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                 // bInterval (10 ms)
	}//;
	,
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	TX_EP3,                    // bEndpointAddress
	BULK_EP,                  // bmAttributes
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                 // bInterval (10ms)
	}//;
	,

	// NOTE:	As per USB 2.0 Sec 5.6.3, the default payload for an
	//		ISO endpoint must be 0x00. Alternate settings should
	//		give alternative values for bandwidth selection
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	RX_EP6,                     // bEndpointAddress
	BULK_EP,                // bmAttributes            
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                  // bInterval (1 for iso endpoints)
	}//;
	,
	{
	sizeof(UsbEndpointDescriptorType), //bLength
	ENDPOINT_DES,                  // descriptor type  
	TX_EP5,                    // bEndpointAddress
	BULK_EP,                // bmAttributes
	0x0040,          // wMaxPacketSize ( Note: 2 bytes wide! )
	0                                  // bInterval (1 for iso endpoints)
	}//;
};

const t_TotalDescriptorType TotalDescriptor[NO_OF_CONFIGURATIONS] =
{
	{(uint8 *) &Configuration1, sizeof (Configuration1)}	// Config 1
};

// Also define number of interfaces each configuration has
const uint8 ucNoOfInterfaces[NO_OF_CONFIGURATIONS] =
{
	0x01 /*,													*/
};

#ifdef __NSC_CRCC__
#pragma reset_options()
#endif

#endif
