/************************************************************************
	File Name : bdb.h															  

	Function:	This file includes definitions for the Bluetooth
				Developemnt Board (BDB).
************************************************************************/

#ifndef BDB_H
#define BDB_H

#include "drivers.h"

#define EEPROM ATMEL_24C16
#define EXTUART NS16552
#define CODEC MC145483

/***********************************************************************
 *
 * On-Board register section
 *
 ***********************************************************************/

# define UART_ADDR  0xfffb10   //For  BDB UART channel I
# define LEDCTRL (* ((FARIO volatile unsigned short *) 0xfffb30))
# define BDBID (* ((FARIO volatile unsigned short *) 0xfffb34))
# define LCDDOUT (* ((FARIO volatile unsigned short *) 0xfffb36))
# define LCDOEN (* ((FARIO volatile unsigned short *) 0xfffb38))
# define LCDDIN (* ((FARIO volatile unsigned short *) 0xfffb3a))
# define KBDDOUT (* ((FARIO volatile unsigned short *) 0xfffb3c))
# define KBDOEN (* ((FARIO volatile unsigned short *) 0xfffb3e))
# define KBDDIN (* ((FARIO volatile unsigned short *) 0xfffb40))
# define DIPSW (* ((FARIO volatile unsigned short *) 0xfffb42))

/***********************************************************************
 *
 * 16552 UART section
 *
 ***********************************************************************/

typedef FARIO union {

	volatile struct {
		unsigned char rhr;
		unsigned char fill1;
		unsigned char fill2;
		unsigned char fill3;
		unsigned char isr;
		unsigned char fill4;
		unsigned char fill5;
		unsigned char fill6;
		unsigned char fill7;
		unsigned char fill8;
		unsigned char lsr;
		unsigned char fill9;
		unsigned char msr;
	} read;

	volatile struct {
		unsigned char thr;
	   	unsigned char fill1;
		unsigned char ier;
		unsigned char fill2;
		unsigned char fcr;
		unsigned char fill3;
		unsigned char lcr;
		unsigned char fill4;
		unsigned char mcr;
	} write;

	volatile struct {
		unsigned char dll;
		unsigned char fill1;
		unsigned char dlm;
	} divisor;

} uart_t;


extern volatile uart_t * const uart;

#define RHR (uart->read.rhr)
#define ISR (uart->read.isr)
#define LSR (uart->read.lsr)
#define MSR (uart->read.msr)

#define THR (uart->write.thr)
#define IER (uart->write.ier)
#define FCR (uart->write.fcr)
#define LCR (uart->write.lcr)
#define MCR (uart->write.mcr)

#define DLL (uart->divisor.dll)
#define DLM (uart->divisor.dlm)

#define INPUT_CLOCK_FREQ 18432000

#define DIVISOR_921600  ((INPUT_CLOCK_FREQ/16)/921600)
#define DIVISOR_115200  ((INPUT_CLOCK_FREQ/16)/115200)
#define DIVISOR_38400   ((INPUT_CLOCK_FREQ/16)/38400)
#define DIVISOR_9600   ((INPUT_CLOCK_FREQ/16)/9600)

#define DIVISOR DIVISOR_115200

/***********************************************************************
 *
 * LED Display section
 *
 ***********************************************************************/

#define LED_TYPE LED_MEM_MAPPED_RW
#define LED_REG LEDCTRL
#define LED_REG_OFFSET 0
#define LED_MASK 0x0007
#define LED_POLARITY 1

#define LED7SEG_TYPE LED7SEG_MEM_MAPPED_RW
#define LED7SEG_REG LEDCTRL
#define LED7SEG_REG_OFFSET 4
#define LED7SEG_MASK 0x00f0
#define LED7SEG_POLARITY 1
#define LED7SEG_DIRECT

/***********************************************************************
 *
 * DIP switch section
 *
 ***********************************************************************/

/* BDB switch numbering scheme:
	   Dip switch S1 1-8 and dip switch S2 1: used for hardware configuration.
	   Dip switch S2 2-8 are logical switches 0-6.
*/

#define NUM_SWITCHES 7
#define SWITCHREG DIPSW
#define SWITCH0_OFFSET 9
#define SWITCH_POLARITY 0
 
/***********************************************************************
 *
 * LCD Display section
 *
 ***********************************************************************/ 
#define     DISP_DATA   (*((FARIO volatile uint8 *)0xFFFB36))
#define     DISP_CTRL   (*((FARIO volatile uint8 *)0xFFFB37))  
#define     DISP_DIR    (*((FARIO volatile uint16 *)0xFFFB38))
#define     DISP_INPT   (*((FARIO volatile uint8 *)0xFFFB3a))                

#define DISP_LEN        24                          /* Length in characters                                 */
#define DISP_LINES      2                           /* Number of lines in display                           */
#define DISP_MAX_CHARS  DISP_LEN * DISP_LINES       /* Total number of characters on display                */

#define DISP_RS         0x01                        /* Display Register select bit                          */
#define DISP_RW         0x02                        /* Display Read/Write bit                               */
#define DISP_EN         0x04                        /* Display Enable (strobe)                              */
#define DISPLAY_BUSY    0x80		                    /* Display Busy signal                                  */

#define DISPLAY_UPDATE  0x01                        /* Various flags used by driver                         */
#define DISPLAY_LINE2   0x02
#define DISPLAY_CMD     0x04


#define CGRAM_ADDRESS   0x50                        /* Starting addresses of the custom char RAM            */
#define LINE1           0x80                        /* Starting addresses of the display lines (display     */
#define LINE2           0xC0                        /* dependent)                                           */


#define DISPLAY_SHIFT   (uint8)0x1C                 /* Display commands. Most are common among displays     */
#define DISPLAY_FORMAT  (uint8)0x38
#define DISPLAY_ON      (uint8)0x0C
#define DISPLAY_MODE    (uint8)0x06
#define DISPLAY_CLEAR   (uint8)0x01
#define DISPLAY_DDHOME  (uint8)0x80

#define FSYS        12000000
#define ONEuS       FSYS/1000000                     /* 1us                                                  */                                         
#define HUNDuS      FSYS/10000                       
#define ONEmS       HUNDuS*10                       


void    DisplayWrite (uint16 line, uint16 pos, uint16 cnt);
void    DisplayPuts  (uint8 line, uint8 pos, uint8 *s);
void    DisplayPutc  (uint8 line, uint8 pos, uint8 c );
void    DisplayInit  (void);
void    DisplayClr   (void);
void    ToggleEN     (void  *dctl);
void    DisplayCtrl  (uint8 , uint16 );
#endif /* BDB_H */
