/******************************************************************************
*  File:	boarddrv.h
*  
*  This is the board level driver API header file.
******************************************************************************/

#ifndef BOARDDRV_H
#define BOARDDRV_H

#include "config.h"
#include "drivers.h"


/***********************************************************************
 *
 * EEPROM section
 *
 ***********************************************************************/void eep_init(void);
extern void eep_buf_write(unsigned short, unsigned char *, unsigned int);
extern void eep_buf_read(unsigned short, unsigned char *, unsigned int);
extern unsigned short eep_total_size(void);


/***********************************************************************
 *
 * LED section
 *
 ***********************************************************************/
extern void led_init(void);
extern void led_7seg_set(unsigned char);
extern void led_set_all(unsigned short);
extern void led_set(unsigned int);
extern void led_clear(unsigned int);
extern void led_toggle(unsigned int);

/***********************************************************************
 *
 * External UART section
 *
 ***********************************************************************/
extern void extuart_init(bitrate_t);
extern unsigned int extuart_tx(unsigned char *, unsigned int);
extern unsigned int extuart_rx(unsigned char *, unsigned int);
extern unsigned int extuart_rx_count(void);


/***********************************************************************
 *
 * Switch section
 *
 ***********************************************************************/
extern boolean switch_on(unsigned int);
extern unsigned int switch_count(void);

/***********************************************************************
 *
 * Codec section
 *
 ***********************************************************************/
extern void codec_init(dl_t *, dl_t *, void (*)(unsigned int), audio_conv_t);
extern void codec_set_volume(unsigned int);
extern void codec_adjust_volume(int);
extern void codec_ctrl( unsigned char addr, unsigned char val );
extern void codec_start(void);
extern void codec_stop(void);

/***********************************************************************
 *
 * RF section
 *
 ***********************************************************************/
extern void rf_set_bbclk(void);
extern void rf_write_reg(char addr, unsigned short data);


/****************************************************************
 *
 * External Flash Section
 *
 ****************************************************************/
 
#if (ISP_EXTERNAL_FLASH == ISP_EXTERNAL_FLASH_CFI)
#	define ISP_CFI_EXTERNAL_FLASH 1
#endif

#ifdef ISP_CFI_EXTERNAL_FLASH

typedef enum {
  CFI_QUERY_ID_STRING,
  CFI_QUERY_SYS_INTERFACE_INFO,
  CFI_QUERY_DEVICE_GEOMETRY,
  CFI_QUERY_PRIM_ALG_EXTENDED,
  CFI_QUERY_SECURITY_CODE
} query_t;

typedef struct cfiQueryIdString {
  uint16 Query[3];
  uint32 PrimAlgCmdSet;
  uint32 AddrPrimAlg;
  uint32 AlterAlgCmdSet;
  uint32 AddrAlterCmdSet;  
} CfiQueryIdString_t;

#define CFI_QUERY_STRING_ADDR 0x10


typedef struct cfiQuerySysInterfaceInfo {
  uint16 MinProgEraseVolt;
  uint16 MaxProgEraseVolt;
  uint16 SupMinProgEraseVolt;
  uint16 SupMaxProgEraseVolt;
  uint16 TimeoutTypicalPerWord;
  uint16 TimeoutTypicalPerBuff;
  uint16 TimeoutTypicalBlockErase;
  uint16 TimeoutTypicalChipErase;
  uint16 TimeoutMaxPerWord;
  uint16 TimeoutMaxPerBuff;
  uint16 TimeoutMaxBlockErase;
  uint16 TimeoutMaxChipErase;
} CfiQuerySysInterfaceInfo_t;

#define CFI_QUERY_SYS_INTERFACE_INFO_ADDR 0x1B

typedef struct cfiQueryDeviceGeometryDefinition {
  uint16 DeviceSize;
  uint32 FlashDeviceInterfaceCodeDescript;
  uint32 MaxNumByteMultiByteProg;
  uint16 NumEraseBlock;
  uint32 R1InfoNumBlock;
  uint32 R1InfoBlockSize;
  uint32 R2InfoNumBlock;
  uint32 R2InfoBlockSize;
  uint32 R3InfoNumBlock;
  uint32 R3InfoBlockSize;
  uint32 R4InfoNumBlock;
  uint32 R4InfoBlockSize;
} CfiQueryDeviceGeometryDefinition_t;

#define CFI_QUERY_DEVICE_GEOMETRY_ADDR 0x27

typedef struct CfiQueryPrimAlgExtended {
  uint16 ExtendedQuery[3];
  uint16 MajorVersion;
  uint16 MinorVersion;
  uint16 AddrSensitiveUnlock;
  uint16 EraseSuspend;
  uint16 BlockProtection;
  uint16 TempBlockUnprotect;
  uint16 BlockProtectUnprotect;
  uint16 SimultaneousOp;
  uint16 BurstMode;
  uint16 PageMode;
} CfiQueryPrimAlgExtended_t;

#define CFI_QUERY_PRIM_ALG_EXTENDED_ADDR 0x40

typedef struct CfiQuerySecurityCodeArea {
  uint16 deviceNumber[4];
} CfiQuerySecurityCodeArea_t;

#define CFI_QUERY_SECURITY_CODE_AREA_ADDR 0x61



typedef enum {
  CFI_FLASH_READ_MANUFACTURER=0x100,
  CFI_FLASH_READ_DEVICE_CODE
} func_t;

typedef enum {
  CFI_FLASH_SUCCESS,
  CFI_FLASH_TOGGLE_FAIL,
  CFI_FLASH_BLOCK_INVALID,
  CFI_FLASH_BLOCK_PROTECTED,
  CFI_FLASH_BLOCK_UNPROTECTED,
  CFI_FLASH_TOO_MANY_BLOCKS, 
  CFI_FLASH_MPU_TOO_SLOW, 
  CFI_FLASH_WRONG_TYPE, 
  CFI_FLASH_ERASE_FAIL, 
  CFI_FLASH_BLOCK_FAILED_ERASE,
  CFI_FLASH_BLOCK_NOT_ERASED,
  CFI_FLASH_BLOCK_ERASED,
  CFI_FLASH_BLOCK_ERASE_FAILURE,
  CFI_FLASH_PROGRAM_FAIL,
  CFI_FLASH_OFFSET_OUT_OF_RANGE
} cfi_result_t;


extern int CfiFlashGetNumBlocks(void);
extern int CfiFlashAutoSelect(func_t func);
extern int CfiFlashReadQuery(query_t query, uint16 *resultBuf);
extern int CfiFlashEraseAll( unsigned char *ucResults );
extern int CfiFlashProgram( unsigned long ulOff, size_t NumWords, void *Array );
extern int CfiFlashPageErase( unsigned char ucNumBlocks, unsigned char ucBlock[] );
extern int CfiFlashBlock(uint8 *address, uint32 numBlock,  uint8 *result);

#endif


#endif

