/************************************************************************
	File Name : cp3bt2x.h															  

	Function:	The file contains peripheral definitions for the CP3BT2X
				processor fmaily. It also includes function prototypes
				for the peripheral drivers.

************************************************************************/

#ifndef CP3BT2X_H
#define CP3BT2X_H

#include "drivers.h"

/***********************************************************************
 *
 * Modules base addresses.
 *
 ***********************************************************************/
//GPIO
#define PB_BASE_ADDR		0xfffb00
#define PC_BASE_ADDR		0xfffb10
#define PE_BASE_ADDR		0xfffcc0
#define PF_BASE_ADDR		0xfffce0
#define PG_BASE_ADDR		0xfff300
#define PH_BASE_ADDR		0xfff320
#define PJ_BASE_ADDR		0xfff340
#define ICU_BASE_ADDR		0xfffe00
#define DMA_BASE_ADDR		0xfff800
#define BIU_BASE_ADDR		0xfff900
#define SYSCFG_BASE_ADDR	0xfff910
#define DATAFSM_BASE_ADDR	0xfff740
#define PROGFM_BASE_ADDR	0xfff940
#define CVSD_BASE_ADDR		0xfffc20
#define TCR_BASE_ADDR		0xfffc40
#define PMM_BASE_ADDR		0xfffc60
#define AAI_BASE_ADDR		0xfffd40
#define MW_BASE_ADDR		0xfff3A0
#define TWM_BASE_ADDR		0xffff20
#define MFT_BASE_ADDR		0xffff40
#define VTU_BASE_ADDR       0xffff80
#define USB_BASE_ADDR		0xfffd80

//
#define USART0_BASE_ADDR	0xfff200
#define USART1_BASE_ADDR	0xfff220
#define USART2_BASE_ADDR	0xfff240
#define USART3_BASE_ADDR	0xfff260
//
#define CAN0_MBUF_BASE_ADDR 0x0ef000
#define CAN0_BASE_ADDR		0x0ef100
#define CAN1_MBUF_BASE_ADDR 0x0ef200
#define CAN1_BASE_ADDR      0x0ef300
//   
#define LPC_BASE_ADDR		0xFFF600
//
#define ACB0_BASE_ADDR		0xfff2a0
#define ACB1_BASE_ADDR		0xfff2c0
//
#define ADC_BASE_ADDR		0xfff3c0
//
#define RNG_BASE_ADDR		0xfff280
//
#define MIWU0_BASE_ADDR		0xfffc80
#define MIWU1_BASE_ADDR		0xfffca0

/***********************************************************************
 *
 * GPIO section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char pxalt;
  unsigned char f1;
  unsigned char pxdir;
  unsigned char f2;
  unsigned char pxdin;
  unsigned char f3;
  unsigned char pxdout;
  unsigned char f4;
  unsigned char pxwkpu;
  unsigned char f5;
  unsigned char pxhdrv;
  unsigned char f6;
  unsigned char pxalts;
} gpio_t;

extern gpio_t * const Bport;
extern gpio_t * const Cport;
extern gpio_t * const Eport;
extern gpio_t * const Fport;
extern gpio_t * const Gport;
extern gpio_t * const Hport;
extern gpio_t * const Jport;

#define PXALT(portnum) portnum##port->pxalt
#define PXDIR(portnum) portnum##port->pxdir
#define PXDIN(portnum) portnum##port->pxdin
#define PXDOUT(portnum) portnum##port->pxdout
#define PXWKPU(portnum) portnum##port->pxwkpu
#define PXHDRV(portnum) portnum##port->pxhdrv
#define PXALTS(portnum) portnum##port->pxalts

/***********************************************************************
 *
 * ICU section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char ivct;
  unsigned char f1;
  unsigned char  nmistat;
  unsigned char  f2;
  unsigned char  exnmi;
  unsigned char  f3;
  unsigned char nminmtr;
  unsigned char  f4;
  unsigned char  f5;
  unsigned char  f6;
  unsigned short istat0;
  unsigned short istat1;
  unsigned short ienam0;
  unsigned short ienam1;
  unsigned short f7;
  unsigned short f8;
  unsigned short f9;
  unsigned short f10;
  unsigned short f11;
  unsigned short f12;
  unsigned short f13;
  unsigned short istat2;
  unsigned short ienam2;
} icu_t;

#define IVCT icu->ivct
#define NMISTAT icu->nmistat
#define EXNMI icu->exnmi
#define NMINMTR icu->nminmtr
#define ISTAT0 icu->istat0
#define ISTAT1 icu->istat1
#define ISTAT2 icu->istat2
#define IENAM0 icu->ienam0
#define IENAM1 icu->ienam1
#define IENAM2 icu->ienam2

#define USART0TX_IRQ		23	
#define MW_IRQ				24
#define VTUD_IRQ            25
#define VTUC_IRQ            26
#define VTUB_IRQ            27
#define VTUA_IRQ            28
#define MFTB_IRQ			29
#define MFTA_IRQ			30
#define ACB1_IRQ			14
#define ACB0_IRQ			31
#define CAN0_IRQ			35
#define CAN1_IRQ			21
#define USART0RX_IRQ		33
#define DMA_CHANNEL3_IRQ	36
#define DMA_CHANNEL2_IRQ	37
#define DMA_CHANNEL1_IRQ	38
#define DMA_CHANNEL0_IRQ	39
#define USB_IRQ				40
#define BT_LLC0_IRQ 		46
#define TWM_IRQ				47
#define AAI_IRQ				34
#define CVSD_IRQ			32

#define ADC_IRQ				13
#define USART1RX_IRQ		20
#define USART1TX_IRQ		19
#define USART2RX_IRQ		18
#define USART2TX_IRQ		17
#define USART3RX_IRQ		16
#define USART3TX_IRQ		15
#define LPC_IRQ				4
#define RNG_IRQ				3


/* Interrupt Dispatch Table Width is defined as the size of a function
   pointer.
*/
#define IDT_WIDTH sizeof(void(*)(void))

extern icu_t * const icu;


/***********************************************************************
 *
 * BIU section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char bcfg;
  unsigned char f1;
  unsigned short iocfg;
  unsigned short szcfg0;
  unsigned short szcfg1;
  unsigned short szcfg2;
} biu_t;

#define BCFG biu->bcfg
#define IOCFG biu->iocfg
#define SZCFG0 biu->szcfg0
#define SZCFG1 biu->szcfg1
#define SZCFG2 biu->szcfg2

/*Bit-wise definition for the BCFG Register*/
#define EWR 0x01
#define OBR 0x02
#define ISTL 0x04
/*Bit-wise definition for the IOCFG/SZCFG Register*/
#define WAIT0 0x0000
#define WAIT1 0x0001
#define WAIT2 0x0002
#define WAIT3 0x0003
#define WAIT4 0x0004
#define WAIT5 0x0005
#define WAIT6 0x0006
#define WAIT7 0x0007
#define HOLD0 0x0000
#define HOLD1 0x0008
#define HOLD2 0x0010
#define HOLD3 0x0018
#define BW 0x0080
#define IPST 0x0200

#define RBE 0x0020
#define WBR 0x0040
#define IPRE 0x0400
#define FRE 0x0800


extern biu_t * const biu;


/***********************************************************************
 *
 * USART section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char utbuf;
  unsigned char f1;
  unsigned char urbuf;
  unsigned char f2;
  unsigned char uictrl;
  unsigned char f3;
  unsigned char ustat;
  unsigned char f4;
  unsigned char ufrs;
  unsigned char f5;
  unsigned char umdsl1;
  unsigned char f6;
  unsigned char ubaud;
  unsigned char  f7;
  unsigned char uspr;
  unsigned char f8;
  unsigned char uovr;
  unsigned char f9;
  unsigned char umdsl2;
  unsigned char f10;
  unsigned char uspos;
} usart_t;

//extern usart_t * const usart;

#define N_USARTS 4
#define USART_BASE_ADDRESSES (usart_t*)USART0_BASE_ADDR, (usart_t*)USART1_BASE_ADDR, \
							(usart_t*)USART2_BASE_ADDR, (usart_t*)USART3_BASE_ADDR

#define USART_MUX_PORTS (gpio_t *)PE_BASE_ADDR, (gpio_t *)PH_BASE_ADDR, \
						(gpio_t *)PH_BASE_ADDR, (gpio_t *)PH_BASE_ADDR
#define USART_MUX_MASK 0x0f, 0x03, 0x0c, 0x30
#define USARTTX_IRQS USART0TX_IRQ, USART1TX_IRQ, USART2TX_IRQ, USART3TX_IRQ
#define USARTRX_IRQS USART0RX_IRQ, USART1RX_IRQ, USART2RX_IRQ, USART3RX_IRQ

#define UTBUF usart->utbuf
#define URBUF usart->urbuf
#define UICTRL usart->uictrl
#define USTAT usart->ustat
#define UFRS usart->ufrs
#define UMDSL1 usart->umdsl1
#define UBAUD usart->ubaud
#define UPSR usart->uspr
#define UOVR usart->uovr
#define UMDSL2 usart->umdsl2
#define USPOS usart->uspos

/*Bit-wise definition for the UFRS Register*/
#define UCHAR_MODE8 0x00
#define UCHAR_MODE7 0x01
#define UCHAR_MODE9 0x02
#define UCHAR_MODE_LOOPBACK9 0x03
#define USTP 0x04
#define UXB9 0x08
#define UPSEL_ODD 0x00
#define UPSEL_EVEN 0x10
#define UPSEL_MARK 0x20
#define UPSEL_SPACE 0x30
#define UPEN 0x80
/*Bit-wise definition for the UMDSL1 Register*/
#define UMOD 0x01
#define UATN 0x02
#define UBRK 0x04
#define UCKS 0x08
#define UETD 0x10
#define UERD 0x20
#define UFCE 0x40
#define URTS 0x80
/*Bit-wise definition for the UMDSL2 Register*/
#define U1SMD 0x01
/*Bit-wise definition for the UICTRL Register*/
#define UTBE 0x01
#define URBF 0x02
#define UDCTS 0x04
#define UCTS 0x08
#define UEFCI 0x10
#define UETI 0x20
#define UERI 0x40
#define UEEI 0x80
/*Bit-wise definition for the USTAT Register*/
#define UPE 0x01
#define UFE 0x02
#define UDOE 0x04
#define UERR 0x08
#define UBKD 0x10
#define URB9 0x20
#define UXMIP 0x40



/***********************************************************************
 *
 * TWM section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char twcfg;
  unsigned char f1;
  unsigned char twcp;
  unsigned char f2;
  unsigned short twmt0;
  unsigned char t0csr;
  unsigned char f3;
  unsigned char wdcnt;
  unsigned char f4;
  unsigned char wdsdm;
} twm_t;

extern twm_t * const twm;

#define TWCFG twm->twcfg
#define TWCP twm->twcp
#define TWMT0 twm->twmt0
#define T0CSR twm->t0csr
#define WDCNT twm->wdcnt
#define WDSDM twm->wdsdm

/*Bit-wise definition for the TWCFG Register*/
#define LTWCFG 0x01
#define LTWCP 0x02
#define LTWMT0 0x04
#define LWDCNT 0x08
#define WDCT0I 0x10
#define WDSDME 0x20
/*Bit-wise definition for the T0CSR Register*/
#define RST 0x01
#define TC 0x02
#define T0INTE 0x04
#define WDTLD 0x08
#define FRZT0E 0x10

/***********************************************************************
 *
 * System Configuration
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned char mcfg;
  unsigned char f1;
  unsigned char dbgcfg;
  unsigned char f2;
  unsigned char mstat;
  unsigned char  f3;
  unsigned char  f4;
  unsigned char  f5;
  unsigned char  swreset;
} syscfg_t;

extern syscfg_t * const syscfg;

#define MCFG syscfg->mcfg
#define DBGCFG syscfg->dbgcfg
#define MSTAT syscfg->mstat
#define SWRESET syscfg->swreset

/* Bit-wise definination for MCFG register*/
#define EXIOE 0x01                 /*IO Expansion Enable*/
#define PLLCLKOE 0x02              /*PLL Clock Output Enable*/
#define MCLKOE 0x04                /*Main Clock Output Enable*/
#define SCLKOE 0x08                /*Slow Clock Output Enable*/
#define USB_ENABLE 0x10	           /*USB enable*/
/* Bit-wise definination for MSTAT register*/
#define OENV0 0x01
#define OENV1 0x02
#define OENV2 0x04
#define PGMBUSY 0x08               /*Flash Programming Busy*/
#define DPGMBUSY 0x10              /*Data Flash Programming Busy*/
#define WDRST   0x40                 /* A watch dog reset occured */
#define ISPRST  0x80              /* an ISP reset occured  */


/***********************************************************************
 *
 * Power Management module section
 *
 ***********************************************************************/
typedef volatile FARIO struct {
  unsigned char pmmcr;
  unsigned char pmmsr;
} pmm_t;

extern pmm_t * const pmm;

#define PMMCR pmm->pmmcr
#define PMMSR pmm->pmmsr

/* Bit-wise definination for PMM register*/
#define HALT 0x04                /* Halt Enable */
#define IDLE 0x02                /* Sysem idle Enable*/
#define PSM 0x01                 /* power save mode Enable */

#define WBPSM 0x08               /* wait before power save mode Enable*/
#define DMC  0x10                /* Disable Main clock Enable*/
#define DHC  0x20                /* Disable PLL clock Enable*/




/***********************************************************************
 *
 * Triple Clock and Reset module section
 *
 ***********************************************************************/


typedef volatile FARIO struct {
  unsigned char crctrl;
  unsigned char f1;
  unsigned char prsfc;
  unsigned char f2;
  unsigned char prssc;
  unsigned char  f3;
  unsigned char prsac;
} tcr_t;

extern tcr_t * const tcr;

#define CRCTRL tcr->crctrl
#define PRSFC tcr->prsfc
#define PRSSC tcr->prssc
#define PRSAC tcr->prsac

/* Triple Clock & Reset module register bits */ 
#define PLLPWD 0x04   /* PLL power down */ 
#define FCLK 0x02     /* Select main clock source */ 
#define SCLK 0x01     /* Select slow clock source */
#define ACE1 0x8      /* Enable Auxiliary clock 1 */
#define ACE2 0x10     /* Enable Auxiliary clock 2 */
#define POR 0x20      /* Power-on reset flag */
 


/***********************************************************************
 *
 * MFT section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned short tcnt1;
  unsigned short tcra;
  unsigned short tcrb;
  unsigned short tcnt2;
  unsigned char tprsc;
  unsigned char f1;
  unsigned char tckc;
  unsigned char f2;
  unsigned char tctrl;
  unsigned char f3;
  unsigned char tictl;
  unsigned char f4;
  unsigned char ticlr;
} mft_t;

extern mft_t * const mft;

#define MFTA_MUX_PORT Gport
#define MFTB_MUX_PORT Eport

#define TCNT1 mft->tcnt1
#define TCRA mft->tcra
#define TCRB mft->tcrb
#define TCNT2 mft->tcnt2
#define TPRSC mft->tprsc
#define TCKC mft->tckc
#define TCTRL mft->tctrl
#define TICTL mft->tictl
#define TICLR mft->ticlr

/*Bit-wise definition for the TCTRL Register*/
#define TMDSEL_MODE1 0x00
#define TMDSEL_MODE2 0x01
#define TMDSEL_MODE3 0x02
#define TMDSEL_MODE4 0x03
#define TAEDG_H_TO_L 0x00
#define TAEDG_L_TO_H 0x04
#define TBEDG_H_TO_L 0x00
#define TBEDG_L_TO_H 0x08
#define TAEN 0x10
#define TBEN 0x20
#define TAOUT 0x40
#define TEN 0x80
/*Bit-wise definition for the TICTL Register*/
#define TAPND 0x01
#define TBPND 0x02
#define TCPND 0x04
#define TDPND 0x08
#define TAIEN 0x10
#define TBIEN 0x20
#define TCIEN 0x40
#define TDIEN 0x80
/*Bit-wise definition for the TICLR Register*/
#define TACLR 0x01
#define TBCLR 0x02
#define TCCLR 0x04
#define TDCLR 0x08
/* MFT operational mode */
#define     TMODE1      0x00                           /* PWM + SYS timer                                      */
#define     TMODE2      0x01
#define     TMODE3      0x02
#define     TMODE4      0x03
#define     STARTC1     0x01
#define     STARTC2     0x08
/* CPU  clock  source*/
#define     FSCLK       50000
#define     FASTCLK_SEL 0x02
#define     SLCLK_PRE   12000000/2/FSCLK - 1
/*Bit-wise definition for the TCKC Register*/
#define C1CSEL_NO_CLK 0x00
#define C1CSEL_PRESCALED_SYS_CLK 0x01
#define C1CSEL_EXT_EVENT 0x02
#define C1CSEL_PULSE_ACCUMULATE 0x03
#define C1CSEL_SLOW_CLK 0x04
#define C2CSEL_NO_CLK 0x00
#define C2CSEL_PRESCALED_SYS_CLK 0x10
#define C2CSEL_EXT_EVENT 0x20
#define C2CSEL_PULSE_ACCUMULATE 0x30
#define C2CSEL_SLOW_CLK 0x40

/***********************************************************************
 *
 * VTU section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
  unsigned short mode;
  unsigned short io1ctl;
  unsigned short io2ctl;
  unsigned short intctl;
  unsigned short intpnd;
  unsigned short clk1ps;
  unsigned short count1;
  unsigned short percap1;
  unsigned short dtycap1;
  unsigned short count2;
  unsigned short percap2;
  unsigned short dtycap2;
  unsigned short clk2ps;
  unsigned short count3;
  unsigned short percap3;
  unsigned short dtycap3;
  unsigned short count4;
  unsigned short percap4;
  unsigned short dtycap4;
} vtu_t;

extern vtu_t * const vtu;

#define VTU_MUX_PORT Fport

#define MODE vtu->mode
#define IO1CTL vtu->io1ctl
#define IO2CTL vtu->io2ctl
#define INTCTL vtu->intctl
#define INTPND vtu->intpnd
#define CLK1PS vtu->clk1ps
#define CLK2PS vtu->clk2ps
#define COUNT1 vtu->count1
#define PERCAP1 vtu->percap1
#define DTYCAP1 vtu->dtycap1
#define COUNT2 vtu->count2
#define PERCAP2 vtu->percap2
#define DTYCAP2 vtu->dtycap2
#define COUNT3 vtu->count3
#define PERCAP3 vtu->percap3
#define DTYCAP3 vtu->dtycap3
#define COUNT4 vtu->count4
#define PERCAP4 vtu->percap4
#define DTYCAP4 vtu->dtycap4

/*Bit-wise definition for the MODE Register*/
#define T1RUN 0x0001
#define T2RUN 0x0002
#define TMOD1_LOW_PWR 0x0000
#define TMOD1_DUAL_8PWM 0x0004
#define TMOD1_16PWM 0x0008
#define TMOD1_CAP 0x000c
#define T3RUN 0x0010
#define T4RUN 0x0020
#define TMOD2_LOW_PWR 0x0000
#define TMOD2_DUAL_8PWM 0x0040
#define TMOD2_16PWM 0x0080
#define TMOD2_CAP 0x00c0
#define T5RUN 0x0100
#define T6RUN 0x0200
#define TMOD3_LOW_PWR 0x0000
#define TMOD3_DUAL_8PWM 0x0400
#define TMOD3_16PWM 0x0800
#define TMOD3_CAP 0x0c00
#define T7RUN 0x1000
#define T8RUN 0x2000
#define TMOD4_LOW_PWR 0x0000
#define TMOD4_DUAL_8PWM 0x4000
#define TMOD4_16PWM 0x8000
#define TMOD4_CAP 0xc000
/*Bit-wise definition for the IO1CTL Register*/
#define C1EDG_REDG_NORESET 0x0000
#define C1EDG_FEDG_NORESET 0x0001
#define C1EDG_REDG_RESET 0x0002
#define C1EDG_FEDG_RESET 0x0003
#define C1EDG_BEDG_NORESET 0x0004
#define C1EDG_BEDG_REDGRESET 0x0005
#define C1EDG_BEDG_FEDGRESET 0x0006
#define C1EDG_BEDG_BEDGRESET 0x0007
#define P1POL_LOW 0x0000
#define P1POL_HIGH 0x0008
#define C2EDG_REDG_NORESET 0x0000
#define C2EDG_FEDG_NORESET 0x0010
#define C2EDG_REDG_RESET 0x0020
#define C2EDG_FEDG_RESET 0x0030
#define C2EDG_BEDG_NORESET 0x0040
#define C2EDG_BEDG_REDGRESET 0x0050
#define C2EDG_BEDG_FEDGRESET 0x0060
#define C2EDG_BEDG_BEDGRESET 0x0070
#define P2POL_LOW 0x0000
#define P2POL_HIGH 0x0080
#define C3EDG_REDG_NORESET 0x0000
#define C3EDG_FEDG_NORESET 0x0100
#define C3EDG_REDG_RESET 0x0200
#define C3EDG_FEDG_RESET 0x0300
#define C3EDG_BEDG_NORESET 0x0400
#define C3EDG_BEDG_REDGRESET 0x0500
#define C3EDG_BEDG_FEDGRESET 0x0600
#define C3EDG_BEDG_BEDGRESET 0x0700
#define P3POL_LOW 0x0000
#define P3POL_HIGH 0x0800
#define C4EDG_REDG_NORESET 0x0000
#define C4EDG_FEDG_NORESET 0x1000
#define C4EDG_REDG_RESET 0x2000
#define C4EDG_FEDG_RESET 0x3000
#define C4EDG_BEDG_NORESET 0x4000
#define C4EDG_BEDG_REDGRESET 0x5000
#define C4EDG_BEDG_FEDGRESET 0x6000
#define C4EDG_BEDG_BEDGRESET 0x7000
#define P4POL_LOW 0x0000
#define P4POL_HIGH 0x8000
/*Bit-wise definition for the IO2CTL Register*/
#define C5EDG_REDG_NORESET 0x0000
#define C5EDG_FEDG_NORESET 0x0001
#define C5EDG_REDG_RESET 0x0002
#define C5EDG_FEDG_RESET 0x0003
#define C5EDG_BEDG_NORESET 0x0004
#define C5EDG_BEDG_REDGRESET 0x0005
#define C5EDG_BEDG_FEDGRESET 0x0006
#define C5EDG_BEDG_BEDGRESET 0x0007
#define P5POL_LOW 0x0000
#define P5POL_HIGH 0x0008
#define C6EDG_REDG_NORESET 0x0000
#define C6EDG_FEDG_NORESET 0x0010
#define C6EDG_REDG_RESET 0x0020
#define C6EDG_FEDG_RESET 0x0030
#define C6EDG_BEDG_NORESET 0x0040
#define C6EDG_BEDG_REDGRESET 0x0050
#define C6EDG_BEDG_FEDGRESET 0x0060
#define C6EDG_BEDG_BEDGRESET 0x0070
#define P6POL_LOW 0x0000
#define P6POL_HIGH 0x0080
#define C7EDG_REDG_NORESET 0x0000
#define C7EDG_FEDG_NORESET 0x0100
#define C7EDG_REDG_RESET 0x0200
#define C7EDG_FEDG_RESET 0x0300
#define C7EDG_BEDG_NORESET 0x0400
#define C7EDG_BEDG_REDGRESET 0x0500
#define C7EDG_BEDG_FEDGRESET 0x0600
#define C7EDG_BEDG_BEDGRESET 0x0700
#define P7POL_LOW 0x0000
#define P7POL_HIGH 0x0800
#define C8EDG_REDG_NORESET 0x0000
#define C8EDG_FEDG_NORESET 0x1000
#define C8EDG_REDG_RESET 0x2000
#define C8EDG_FEDG_RESET 0x3000
#define C8EDG_BEDG_NORESET 0x4000
#define C8EDG_BEDG_REDGRESET 0x5000
#define C8EDG_BEDG_FEDGRESET 0x6000
#define C8EDG_BEDG_BEDGRESET 0x7000
#define P8POL_LOW 0x0000
#define P8POL_HIGH 0x8000
/*Bit-wise definition for the INTCTL Register*/
#define I1AEN 0x0001
#define I1BEN 0x0002
#define I1CEN 0x0004
#define I1DEN 0x0008
#define I2AEN 0x0010
#define I2BEN 0x0020
#define I2CEN 0x0040
#define I2DEN 0x0080
#define I3AEN 0x0100
#define I3BEN 0x0200
#define I3CEN 0x0400
#define I3DEN 0x0800
#define I4AEN 0x1000
#define I4BEN 0x2000
#define I4CEN 0x4000
#define I4DEN 0x8000
/*Bit-wise definition for the INTPND Register*/
#define I1APD 0x0001
#define I1BPD 0x0002
#define I1CPD 0x0004
#define I1DPD 0x0008
#define I2APD 0x0010
#define I2BPD 0x0020
#define I2CPD 0x0040
#define I2DPD 0x0080
#define I3APD 0x0100
#define I3BPD 0x0200
#define I3CPD 0x0400
#define I3DPD 0x0800
#define I4APD 0x1000
#define I4BPD 0x2000
#define I4CPD 0x4000
#define I4DPD 0x8000

/***********************************************************************
 *
 * Microwire/SPI section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
	unsigned short mwdat;
	unsigned short mwctl1;
	unsigned short mwstat;
	unsigned short mwctl2;
	unsigned short mwtest;
}mw_t;

#define MW_MUX_PORT Fport
#define MW_MUX_MASK 0x0f

extern mw_t * const mw;

#define MWDAT mw->mwdat
#define MWCTL1 mw->mwctl1
#define MWSTAT mw->mwstat
#define MWCTL2 mw->mwctl2
#define MWTEST mw->mwtest



#define MWEN 0x1
#define MNS 0x2
#define EIR 0x20
#define EIW	0x40
#define BSY 0x1 
#define RBF 0x2



/***********************************************************************
 *
 * Access.bus section
 *
 ***********************************************************************/

typedef volatile FARIO struct {
	unsigned char acbsda;
	unsigned char f1;
	unsigned char acbst;
	unsigned char f2;
	unsigned char acbcst;
	unsigned char f3;
	unsigned char acbctl1;
	unsigned char f4;
	unsigned char acaddr;
	unsigned char f5;
	unsigned char acbctl2;
	unsigned char f6;
	unsigned char acbaddr2;
	unsigned char f7;
	unsigned char acbctl3;
	unsigned char f8;
}acb_t;

//extern acb_t * const acb;

#define N_ACBS 2
#define ACB_BASE_ADDRESSES (acb_t*)ACB0_BASE_ADDR, (acb_t*)ACB1_BASE_ADDR

#define ACB_IRQS ACB0_IRQ, ACB1_IRQ

#define ACBSDA acb->acbsda
#define ACBST acb->acbst
#define ACBCST acb->acbcst
#define ACBCTL1 acb->acbctl1
#define ACBADDR acb->acaddr
#define ACBCTL2 acb->acbctl2
#define ACBADDR2 acb->acbaddr2
#define ACBCTL3 acb->acbctl3

/*Bit-wise definition for the ACBST Register*/
#define XMIT 0x01
#define MASTER 0x02
#define NMATCH 0x04
#define STASTR 0x08
#define NEGACK 0x10
#define BER 0x20
#define SDAST 0x40
#define SLVSTP 0x80
/*Bit-wise definition for the ACBCST Register*/
#define BUSY 0x01
#define BB 0x02
#define MATCH 0x04
#define GCMTCH 0x08
#define TSDA 0x10
#define TGSCL 0x20
#define MATCHAF 0x40
#define ARPMATCH 0x80
/*Bit-wise definition for the ACBCTL1 Register*/
#define START 0x01
#define STOP 0x02
#define INTEN 0x04
#define ACK 0x10
#define GCMEN 0x20
#define NMINTE 0x40
#define STASTRE 0x80
/*Bit-wise definition for the ACBCTL2 Register*/
#define ENABLE 0x01
/*Bit-wise definition for the ACBCTL3 Register*/
#define ARPMEN 0x04
/*Bit-wise definition for the ACBADDR and ACBADDR2 Registers*/
#define SAEN 0x80

/***********************************************************************
 *
 * Random Number Generator (RNG) section
 *
 ***********************************************************************/
 typedef volatile FARIO struct {
	uint16	rngcst;
	uint16	rngd;
	uint16	rngdivh;
	uint16	rngdivl;
 } rng_t;

extern rng_t * const rng;

#define RNGCST	rng->rngcst
#define RNGD	rng->rngd
#define RNGDIVH	rng->rngdivh
#define RNGDIVL	rng->rngdivl

#define RNGE	0x01
#define DVALID	0x02
#define RNG_IMASK	0x20
#define RNGDIV1716	0x03


/***********************************************************************
 *
 * DMA section
 *
 ***********************************************************************/

typedef volatile FARIO struct
{
  void FAR * adca0;	
  void FAR * adra0;	
  void FAR * adcb0;	
  void FAR * adrb0;	
  unsigned short bltc0;	
  unsigned short f1;	
  unsigned short bltr0;	
  unsigned short f2;	
  unsigned long f20;	
  unsigned short dmacntl0;	
  unsigned char dmastat0;	
  unsigned char f3;
  void FAR * adca1;	
  void FAR * adra1;	
  void FAR * adcb1;	
  void FAR * adrb1;	
  unsigned short bltc1;	
  unsigned short f4;	
  unsigned short bltr1;	
  unsigned short f5;	
  unsigned long f21;	
  unsigned short dmacntl1;
  unsigned char dmastat1;	
  unsigned char f6;
  void FAR * adca2;	
  void FAR * adra2;	
  void FAR * adcb2;	
  void FAR * adrb2;	
  unsigned short bltc2;	
  unsigned short f7;	
  unsigned short bltr2;	
  unsigned short f8;	
  unsigned long f22;	
  unsigned short dmacntl2;
  unsigned char dmastat2;
  unsigned char f9;
  void FAR * adca3;	
  void FAR * adra3;	
  void FAR * adcb3;	
  void FAR * adrb3;	
  unsigned short bltc3;	
  unsigned short f10;	
  unsigned short bltr3;	
  unsigned short f11;	
  unsigned long f23;	
  unsigned short dmacntl3;	
  unsigned char dmastat3;	
  unsigned char f12;
} dma_t;

extern dma_t * const dma;

#define ADCA0 dma->adca0
#define ADRA0 dma->adra0
#define ADCB0 dma->adcb0
#define ADRB0 dma->adrb0
#define BLTC0 dma->bltc0
#define BLTR0 dma->bltr0
#define DMACNTL0 dma->dmacntl0
#define DMASTAT0 dma->dmastat0
#define ADCA1 dma->adca1
#define ADRA1 dma->adra1
#define ADCB1 dma->adcb1
#define ADRB1 dma->adrb1
#define BLTC1 dma->bltc1
#define BLTR1 dma->bltr1
#define DMACNTL1 dma->dmacntl1
#define DMASTAT1 dma->dmastat1
#define ADCA2 dma->adca2
#define ADRA2 dma->adra2
#define ADCB2 dma->adcb2
#define ADRB2 dma->adrb2
#define BLTC2 dma->bltc2
#define BLTR2 dma->bltr2
#define DMACNTL2 dma->dmacntl2
#define DMASTAT2 dma->dmastat2
#define ADCA3 dma->adca3
#define ADRA3 dma->adra3
#define ADCB3 dma->adcb3
#define ADRB3 dma->adrb3
#define BLTC3 dma->bltc3
#define BLTR3 dma->bltr3
#define DMACNTL3 dma->dmacntl3
#define DMASTAT3 dma->dmastat3

/*Bit-wise definition for the DMACNTL0,DMACNTL1,DMACNTL2,DMACNTL3 Registers*/
#define CHEN 0x0001
#define ETC 0x0002
#define EOVR 0x0004
#define TCS 0x0008
#define IND 0x0010
#define DIR 0x0020
#define OT 0x0040
#define BPC 0x0080
#define SWRQ 0x0100
#define ADA 0x0200
#define INCA_ADD_1 0x0000
#define INCA_ADD_2 0x0400
#define INCA_DEC_1 0x0800
#define INCA_DEC_2 0x0C00
#define ADB 0x1000
#define INCB_ADD_1 0x0000
#define INCB_ADD_2 0x0200
#define INCB_DEC_1 0x0400
#define INCB_DEC_2 0x0600
/*Bit-wise definition for the DMASTAT0,DMASTAT1,DMASTAT2,DMASTAT3 Registers*/
#define TERMCOUNT 0x01
#define OVR 0x02
#define CHAC 0x04
#define VLD 0x08

/* bit-wise defination for DMA registers  */
#define     CHEN       0x0001
#define     ETC        0x0002
#define     EOVR       0x0004
#define     TCS        0x0008
#define     IND        0x0010
#define     TDIR       0x0020
#define     OPTYP      0x0040
#define     BPC        0x0080
#define     SWRQ       0x0100
#define     ADA        0x0200
#define     INCA0      0x0400
#define     INCA1      0x0800
#define     ADB        0x1000
#define     INCB0      0x2000
#define     INCB1      0x4000

#define     TERMCNT    0x0001
#define     CHOVR      0x0002
#define     CHACT      0x0004



/***********************************************************************
 *
 * USB section
 *
 ***********************************************************************/
  
#define USB_MAX_PKT_LEN 64

// Define the USB_COMPACT symbol for the COMPACT version of the driver, 
// defining it reduces the driver to minimum functionality - it will still
 // enumerate, configure and transfer data
 // Does not support:
 // - callbacks, multiple configurations or interfaces
 // - suspend/resume, clear feature
 // - set descriptor, set feature
 // - usb_shutdown() and get_ep_stat() have null bodies

 /* Used to handle data PIDS for the endpoints */
#define EP_BIT(ep) (1<<(ep))

// These two structs are overlayed over each other to give access to the
// endpoints with the different indexes
typedef volatile struct {
	uint8 epc;
	uint8 f1;
	uint8 dat;
	uint8 f2;
	uint8 stat;
	uint8 f3;
	uint8 cmd;
	uint8 f4;
}	std_ep_ctrl_t;

typedef volatile struct {
	uint8 tx_epc;
	uint8 f1;
	uint8 txd;
	uint8 f2;
	uint8 txs;
	uint8 f3;
	uint8 txc;
	uint8 f4;

	uint8 rx_epc;
	uint8 f5;
	uint8 rxd;
	uint8 f6;
	uint8 rxs;
	uint8 f7;
	uint8 rxc;
	uint8 f8;
}	rxtx_ep_ctrl_t;  

typedef volatile FARIO struct {
	uint8 mcntrl;
	uint8 f1;	
	uint8 xcvrdiag;
	uint8 f2;
	uint8 tcr;
	uint8 f3;
	uint8 utr;
	uint8 f4;
	uint8 far;
	uint8 f5;
	uint8 nfsr;
	uint8 f6;
	uint8 maev;
	uint8 f7;
	uint8 mamsk;
	uint8 f8;

	uint8 altev;   
	uint8 f9;
	uint8 altmask;
	uint8 f10;
	uint8 txev;
	uint8 f11;
	uint8 txmsk;
	uint8 f12;
	uint8 rxev;
	uint8 f13;
	uint8 rxmsk;
	uint8 f14;
	uint8 nakev;
	uint8 f15;
	uint8 nakmsk; 
	uint8 f16;
	uint8 fwev;
	uint8 f17;
	uint8 fwmsk;
	uint8 f18;
	uint8 fnh;
	uint8 f19;
	uint8 fnl;
	uint8 f20;

	uint8 dmacntrl;  
	uint8 f21;
	uint8 dmaev;
	uint8 f22;
	uint8 dmamsk;
	uint8 f23;
	uint8 mir;
	uint8 f24;
	uint8 dmacnt;
	uint8 f25;
	uint8 dmaerr;
	uint8 f26a;
	uint8 f26b[12];
	
	// This is an attempt at getting fairly homogenous acces to the EP control
	// registers when the HW registers don't make it easy to use array indexes
	union { 
		// This one gives ep0 access without runtime offset calcs
		struct {
			rxtx_ep_ctrl_t ep0_ctrl;
			rxtx_ep_ctrl_t ep1to6dummy[3];
		} ep0_xs;

		// This one gives indexing for EPs 1-6
		struct { 
			// NOTE: Do NOT use this struct for EP0, 
			// alignment only works for EPs 1-6, ie ep_ctrl[1] is TX1
			std_ep_ctrl_t dummy;
			std_ep_ctrl_t ep_ctrl[7];
		} epx_xs;

		// This one gives RX/TX0..3 indexing
		rxtx_ep_ctrl_t rxtx_ep_ctrl[4]; // Don't use [0].rx_epc
	} ep_xs;
} usb_t;

extern usb_t * const usb;

#define MCNTRL          usb->mcntrl
#define XCVRDIAG        usb->xcvrdiag 
#define TCR	            usb->tcr 
#define UTR	            usb->utr  
#define FARREG          usb->far
#define NFSR            usb->nfsr 
#define MAEV            usb->maev 
#define MAMSK           usb->mamsk 
 
#define ALTEV           usb->altev 
#define ALTMSK          usb->altmask 
#define TXEV            usb->txev 
#define TXMSK           usb->txmsk  
#define RXEV            usb->rxev 
#define RXMSK           usb->rxmsk 
#define NAKEV           usb->nakev 
#define NAKMSK          usb->nakmsk
 
#define FWEV            usb->fwev
#define FWMSK           usb->fwmsk
#define FNH             usb->fnh
#define FNL             usb->fnl
#define DMACNTRL        usb->dmacntrl
#define DMAEV           usb->dmaev
#define DMAMSK          usb->dmamsk
#define MIR             usb->mir
 
#define DMACNT          usb->dmacnt
#define DMAERR          usb->dmaerr

/* Receive Endpoint0 */  
#define EPC0            usb->ep_xs.ep0_xs.ep0_ctrl.tx_epc 
#define TXD0            usb->ep_xs.ep0_xs.ep0_ctrl.txd 
#define TXS0            usb->ep_xs.ep0_xs.ep0_ctrl.txs 
#define TXC0            usb->ep_xs.ep0_xs.ep0_ctrl.txc
// For EP0 Rx and TX share EPC0 above
#define RXD0            usb->ep_xs.ep0_xs.ep0_ctrl.rxd
#define RXS0            usb->ep_xs.ep0_xs.ep0_ctrl.rxs
#define RXC0            usb->ep_xs.ep0_xs.ep0_ctrl.rxc

// Two ways of accessing the EPs are setup by overlaying two different structs 
// over the control registers.
// Here refer to EPs 1-6 in a homogenous manner, both TX and RX
// DO NOT use ep = 0 here
#define EPC(ep)				usb->ep_xs.epx_xs.ep_ctrl[ep].epc
#define EP_DAT(ep)			usb->ep_xs.epx_xs.ep_ctrl[ep].dat
#define EP_STAT(ep)			usb->ep_xs.epx_xs.ep_ctrl[ep].stat
#define EP_CMD(ep)			usb->ep_xs.epx_xs.ep_ctrl[ep].cmd

// Here refer to TX/RX EPs 0-3
#define TXEPC(ep)          usb->ep_xs.rxtx_ep_ctrl[ep].tx_epc 
#define TXD(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].txd
#define TXS(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].txs
#define TXC(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].txc
// Can't use RXEPC for 0
#define RXEPC(ep)          usb->ep_xs.rxtx_ep_ctrl[ep].rx_epc 
#define RXD(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].rxd
#define RXS(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].rxs
#define RXC(ep)            usb->ep_xs.rxtx_ep_ctrl[ep].rxc

 
/* MCNTRL bits ********************************************************/ 
#define     USBEN      0x01                 /*software reset          */ 
#define     DBG        0x02                 /*debug mode              */ 
#define     VGE        0x04                 /*voltage regulator enable*/ 
#define     NAT        0x08                 /*node attached           */ 
#define     INT_DIS    0x00                 /*interrupts disabled     */ 
#define     INT_L_O    0x40                 /*act lo ints, open drain */ 
#define     INT_H_P    0x80                 /*act hi ints, push pull  */ 
#define     INT_L_P    0xC0                 /*act lo ints, push pull  */ 

/* UTR    bits ********************************************************/
#define		DIAG	   0x80					/* Turns on USB diagnostic*/

/* TCR    bits ********************************************************/ 
#define		CADJ	   0x1F					/* Current Adjust Reg	  */
#define		VADJ	   0xE0					/* Voltage Adjust Reg	  */

/* CCONF  bits ********************************************************/ 
#define     CODIS      0x80                 /*disable clock output    */ 
 
/* FAR    bits ********************************************************/ 
#define     AD_EN      0x80                 /*address enable          */ 
 
/* EPCX   bits ********************************************************/ 
#define     DEF        0x40                 /*force def. adr (0 only) */ 
#define     STALL      0x80                 /*force stall handshakes  */ 
#define     ISO        0x20                 /*set for isochr. (1-3)   */ 
#define     EP_EN      0x10                 /*enables endpt.  (1-3)   */ 
 
/* NFSR   bits ********************************************************/ 
#define     RST_ST     0x00                 /*reset       state       */ 
#define     RSM_ST     0x01                 /*resume      state       */ 
#define     OPR_ST     0x02                 /*operational state       */ 
#define     SUS_ST     0x03                 /*suspend     state       */ 
 
/* MAEV, MAMSK bits ***************************************************/ 
#define     WARN       0x01                 /*warning bit has been set*/ 
#define     ALT        0x02                 /*alternate event         */ 
#define     TX_EV      0x04                 /*transmit event          */ 
#define     FRAME      0x08                 /*SOF packet received     */ 
#define     NAK        0x10                 /*NAK event               */ 
#define     ULD        0x20                 /*unlock locked detected  */ 
#define     RX_EV      0x40                 /*receive event           */ 
#define     INTR_E     0x80                 /*master interrupt enable */ 
 
/* ALTEV, ALTMSK bits *************************************************/ 
#define		DMA		   0x04					/*dma event				  */
#define     EOP        0x08                 /*end of packet           */ 
#define     SD3        0x10                 /*3 ms suspend            */ 
#define     SD5        0x20                 /*5 ms suspend            */ 
#define     RESET_A    0x40                 /*reset  detected         */ 
#define     RESUME_A   0x80                 /*resume detected         */ 
 
/* TXEV, TXMSK bits ***************************************************/ 
#define     TXFIFO0    0x01                 /*TX_DONE, FIFO 0         */ 
#define     TXFIFO1    0x02                 /*TX_DONE, FIFO 1         */ 
#define     TXFIFO2    0x04                 /*TX_DONE, FIFO 2         */ 
#define     TXFIFO3    0x08                 /*TX_DONE, FIFO 3         */ 
#define     TXUDRN0    0x10                 /*TX_URUN, FIFO 0         */ 
#define     TXUDRN1    0x20                 /*TX_URUN, FIFO 1         */ 
#define     TXUDRN2    0x40                 /*TX_URUN, FIFO 2         */ 
#define     TXUDRN3    0x80                 /*TX_URUN, FIFO 3         */ 
 
/* RXEV, RXMSK bits ***************************************************/ 
#define     RXFIFO0    0x01                 /*RX_DONE, FIFO 0         */ 
#define     RXFIFO1    0x02                 /*RX_DONE, FIFO 1         */ 
#define     RXFIFO2    0x04                 /*RX_DONE, FIFO 2         */ 
#define     RXFIFO3    0x08                 /*RX_DONE, FIFO 3         */ 
#define     RXOVRN0    0x10                 /*RX_OVRN, FIFO 0         */ 
#define     RXOVRN1    0x20                 /*RX_OVRN, FIFO 1         */ 
#define     RXOVRN2    0x40                 /*RX_OVRN, FIFO 2         */ 
#define     RXOVRN3    0x80                 /*RX_OVRN, FIFO 3         */ 
 
/* NAKEV, NAKMSK bits *************************************************/ 
#define     NAK_I0     0x01                 /*IN  NAK, FIFO 0         */ 
#define     NAK_I1     0x02                 /*IN  NAK, FIFO 1         */ 
#define     NAK_I2     0x04                 /*IN  NAK, FIFO 2         */ 
#define     NAK_I3     0x08                 /*IN  NAK, FIFO 3         */ 
#define     NAK_O0     0x10                 /*OUT NAK, FIFO 0         */ 
#define     NAK_O1     0x20                 /*OUT NAK, FIFO 1         */ 
#define     NAK_O2     0x40                 /*OUT NAK, FIFO 2         */ 
#define     NAK_O3     0x80                 /*OUT NAK, FIFO 3         */ 
 
/* TXCn   bits ********************************************************/ 
#define     TX_EN      0x01                 /*transmit enable         */ 
#define     TX_LAST    0x02                 /*last data in FIFO       */ 
#define     TX_TOGL    0x04                 /*specifies PID used      */ 
#define     TX_FLUSH   0x08                 /*flushes all FIFO data   */ 
#define		RFF		   0x10					/*refill FIFO TXC1,2,3	  */
#define     IGN_IN     0x10                 /*ignore in tokens EP0    */
#define		TFWL	   0x60					/*tx fifo warning level	  */
#define		IGN_ISOMSK 0x80					/*ignore iso mask		  */
 
/* TXSn   bits ********************************************************/ 
#define		TCOUNT	   0x1F					/*transmission count	  */
#define     TX_DONE    0x20                 /*transmit done           */ 
#define     ACK_STAT   0x40                 /*ack status of xmission  */
#define		TX_URUN	   0x80					/*FIFO underrun			  */
 
/* RXCn   bits ********************************************************/ 
#define     RX_EN      0x01                 /*receive  enable         */ 
#define     IGN_OUT    0x02                 /*ignore out tokens       */ 
#define     IGN_SETUP  0x04                 /*ignore setup tokens     */ 
#define		RX_FLUSH   0x08					/*flush endpoint		  */
#define		RFWL	   0x60					/*refill fifo warning	  */
 
/* RXSn   bits ********************************************************/ 
#define		RCOUNT	   0x0F					/*reception count		  */
#define     RX_LAST    0x10                 /*indicates RCOUNT valid  */ 
#define     RX_TOGL    0x20                 /*last pkt was DATA1 PID  */ 
#define     SETUP_R    0x40                 /*setup packet received   */ 
#define     RX_ERR     0x80                 /*last packet had an error*/ 

/* DMACNTRL bits */
#define		DEN 0x80
#define		IGNRXTGL 0x40
#define		DTGL 0x20
#define		ADMA 0x10
#define		DMOD 0x08
#define		DSRC1 0x00
#define		DSRC2 0x01
#define		DSRC3 0x02
#define		DSRC4 0x03
#define		DSRC5 0x04
#define		DSRC6 0x05

// API for calling usb.c from usbhw.c
void set_device_state( usb_dev_state_t new_dev_state );
void handle_std_usb_request( usb_request_t USBRequest );
void set_remote_wake_ok( boolean wake_ok );
void indicate_suspended( boolean indic_susp );

// API for calling usbhw.c from usb.c
extern void node_attach(void);
extern void node_detach(void);
void reset_ep_pids( void );
void clear_ep_stall(ep_id_t ep); 
void stall_ep0_send00( void );
void resume_endpoint (void);
boolean usb_hwinit( usb_init_info_t *usbinitinfo );
void setup_endpoints (uint8 ucConfiguration, 
        uint8 ucInterface, uint8 ucAltSetting);

extern INTERRUPT void usb_isr(void);
extern void set_iso_synch_frame (uint8, uint16);


/**********************************************************************
 * Advanced Audio Interface	and CVSD conversion section
 * 
 ***********************************************************************/
typedef volatile FARIO struct {
    uint16  ARfr;
    uint16  ARdr0;
    uint16  ARdr1;
    uint16  ARdr2;
    uint16  ARdr3;
    uint16  ATfr;
    uint16  ATdr0;
    uint16  ATdr1;
    uint16  ATdr2;
    uint16  ATdr3;
    uint16  Agcr;
    uint16  Aiscr;
    uint16  ARscr;
    uint16  ATscr;
    uint16  Accr;
    uint16  ADMAcr;
} aai_t;

#define AAI_MUX_PORT Fport
#define AAI_MUX_MASK 0xf0

#define ARFR	    aai->ARfr       
#define ARDR0       aai->ARdr0
#define ARDR1       aai->ARdr1
#define ARDR2       aai->ARdr2
#define ARDR3       aai->ARdr3
#define ATFR        aai->ATfr
#define ATDR0       aai->ATdr0
#define ATDR1       aai->ATdr1
#define ATDR2       aai->ATdr2
#define ATDR3       aai->ATdr3
#define ADMACR      aai->ADMAcr
#define AGCR		aai->Agcr
#define AISCR		aai->Aiscr
#define ARSCR		aai->ARscr
#define ATSCR		aai->ATscr
#define ACCR		aai->Accr
#define ADMACR		aai->ADMAcr

extern aai_t * const aai;

/* AGCR bit definitions */
#define ASS         0x0001
#define DWL         0x0002
#define LPB         0x0004
#define SCS_MASK    0x0018 /* Network Slot select mask*/
#define SCS1        0x0000 /* slot count select - 1 slot/frame */
#define SCS2        0x0008 /* slot count select - 2 slots/frame*/
#define SCS3        0x0010 /* slot count select - 3 slots/frame*/
#define SCS4        0x0018 /* slot count select - 4 slots/frame*/
#define IEFS        0x0020
#define FSS         0x0040
#define IEBC        0x0080
#define CRF         0x0100
#define CTF         0x0200
#define FSL13       0x0000
#define FSL14       0x0400
#define FSL15       0x0800
#define FSL16       0x0C00
#define IFS         0x1000
#define IOM2        0x2000
#define AAIEN       0x4000
#define AACLKEN     0x8000

/* AISCR register bit-wise definations*/

#define RXIE   0x01
#define RXEIE  0x02
#define TXIE   0x04
#define TXEIE  0x08
#define RXIP   0x10
#define RXEIP  0x20
#define TXIP   0x40
#define TXEIP  0x80
#define RXIC   0x0100
#define RXEIC  0x0200
#define TXIC   0x0400
#define TXEIC  0x0800

/* ARSCR register bit-wise defination */

#define RXAF    0x01
#define RXF     0x02
#define RXE     0x04
#define RXO     0x08

#define RXSA0   0x0010
#define RXSA1   0x0020
#define RXSA2   0x0040
#define RXSA3   0x0080

#define RXDSA0    0x0100  /* Slot 0 is assigned to DMA*/
#define RXDSA1    0x0200  /* Slot 1 is assigned to DMA*/
#define RXDSA2    0x0400  /* Slot 2 is assigned to DMA*/
#define RXDSA3    0x0800  /* Slot 3 is assigned to DMA*/

#define RXDSA_NONE  0x0000  /* No slots are assigned */
#define RXDSA_SLOT01 0x0300 /* Slots 0,1 are assigned to DMA*/
#define RXDSA_SLOT12 0x0600 /* Slots 1,2 are assigned to DMA*/
#define RXDSA_SLOT23 0x0C00 /* Slots 2,3 are assigned to DMA*/

#define RXFWM0  0x20
#define RXFWM1  0x20
#define RXFWM2  0x20
#define RXFWM3  0x20

/*  ATSCR register bit-wise defination */
#define TXAE    0x01
#define TXE_A   0x02
#define TXF     0x04
#define TXU     0x08

#define TXSA0   0x0010
#define TXSA1   0x0020
#define TXSA2   0x0040
#define TXSA3   0x0080

#define TXDSA0  RXDSA0
#define TXDSA1  RXDSA1
#define TXDSA2  RXDSA2
#define TXDSA3  RXDSA3

#define TXDSA_NONE   0x0000 /* No slots are assigned */
#define TXDSA_SLOT01 0x0300 /* Slots 0,1 are assigned to DMA */
#define TXDSA_SLOT12 0x0600 /* Slots 1,2 are assigned to DMA*/
#define TXDSA_SLOT23 0x0C00 /* Slots 2,3 are assigned to DMA*/

#define TXFWM0  0x02
#define TXFWM1  0x02
#define TXFWM2  0x02
#define TXFWM3  0x02

/* ACCR register bit-wise definations */

#define CSS     0x01

/* ADMACR */
#define RMD0    0x01
#define RMD1    0x02
#define RMD2    0x04
#define RMD3    0x08
#define TMD0    0x10
#define TMD1    0x20
#define TMD2    0x40
#define TMD3    0x80

/* Need to be reedited */
#define RMD_NONE 0x00					/* No buffer full */
#define RMD_12  0x06					/* ARDR1,2 full */
#define RMD_23  0x0C					/* ARDR1,2 full */

#define TMD_NONE   0x00			  /* No buffer full */
#define TMD_12  0x60	  			/* ATDR1,2 full */
#define TMD_23  0xC0


#define ACD0   0x02


typedef volatile FARIO struct {
    uint16  CVSdin;
    uint16  CVSdout;
    uint16  PCMin;
    uint16  PCMout;
    uint16  LOGin;
    uint16  LOGout;
    uint16  LINin;
    uint16  LINout;
    uint16  CVctrl;
    uint16  CVstat;
    uint16  CVtest;
    uint16  CVradd;
    uint16  CVrdat;
    uint16  CVdecout;
    uint16  CVencin;
    uint16  CVencpr;
} cvsd_t;

extern cvsd_t * const cvsd;

#define     CVSDIN     cvsd -> CVSdin
#define     CVSDOUT    cvsd -> CVSdout
#define     PCMIN      cvsd -> PCMin
#define     PCMOUT     cvsd -> PCMout
#define     LOGIN      cvsd -> LOGin
#define     LOGOUT     cvsd -> LOGout
#define     LINEARIN   cvsd -> LINin
#define     LINEAROUT  cvsd -> LINout
#define     CVCTRL     cvsd -> CVctrl
#define     CVSTAT     cvsd -> CVstat
#define     CVTEST     cvsd -> CVtest
#define     CVRADD     cvsd -> CVradd
#define     CVRDAT     cvsd -> CVrdat
#define     CVDECOUT   cvsd -> CVdecout
#define     CVENCIN    cvsd -> CVencin
#define     CVENCPR    cvsd -> CVencpr

/*Bit-wise for CVCTRL*/
#define     CVEN        0x0001
#define     CVCLKEN     0x0002
#define     PCMINT      0x0004
#define     CVSDINT     0x0008
#define     CVSDERRINT  0x0010
#define     DMACO       0x0020
#define     DMACI       0x0040
#define     DMAPO       0x0080
#define     DMAPI       0x0100

#define     CVSDuL      0x0000				/* u-law PCM */
#define     CVSDaL      0x0200				/* A-law PCM */
#define     CVSDLin     0x0400				/* Linear PCM*/

#define     PCMCONV     0x0800

#define     RES_MASK    (0x3000)
#define     RES16       0x0000				/* 16-bit linear PCM */
#define     RES15       0x1000				/* 15-bit linear PCM */
#define     RES14       0x2000				/* 14-bit linear PCM */
#define     RES13       0x3000				/* linear 13-bit PCM */

/* Bit-wise for CVSTAT*/
#define     CVNE        0x01
#define     CVNF        0x02
#define     PCMINTP     0x04
#define     CVE         0x08
#define     CVF         0x10
#define     CVINST      0xe0
#define     CVOUTST     0x700


/***********************************************************************
 *
 * Bluetooth LLC section
 *
 ***********************************************************************/

#define BT_SEQ_RESET (*((volatile unsigned char *) 0xef1b6))
#define BT_SYS_CLK_ENABLE (*((volatile unsigned char *) 0xef1be))

#define BTLLC_DATA_RAM_ADDR 0xE8000
#define BTLLC_DATA_RAM_SIZE 0x1200

#define BTLLC_MUX_PORT PORTG
#define BTLLC_MUX_MASK 0x3f

#define CCB_SCLK		0x08
#define CCB_SDAT		0x10
#define CCB_SLEN		0x20
#define CCB_MASK		(CCB_SCLK|CCB_SDAT|CCB_SLEN)
 
/****************************************************************
 *
 * Flash Programming Section
 *
 ****************************************************************/

/*********************************
 * Flash Program memory Section *
 *********************************/
typedef volatile FARIO struct {
    uint16  fmibar;
    uint16  fmibdr;
    uint16  fm0wer;
    uint16  fm1wer;
    uint16  fm2wer;
    uint16  fm3wer;
	uint16  fmctrl;
    uint16  fmstat;
    uint8  fmpsr;
    uint8  f1;
    uint8  fmstart;
    uint8  f2;
    uint8  fmtran;
    uint8  f3;
    uint8  fmprog;
    uint8  f4;
    uint8  fmperase;
    uint8  f5;
    uint8  fmmerase0;
    uint8  f6;
    uint8  fmmerase1;
    uint8  f7;
    uint8  fmend;
    uint8  f8;
    uint8  fmmend;
    uint8  f9;
	uint8  fmrcv;
    uint8  f10;
	uint16  fmar0;
	uint16  fmar1;
	uint16  fmar2;

} fm_t;

extern fm_t * const fm;
#define FMIBAR fm-> fmibar
#define FMIBDR fm-> fmibdr
#define	FM0WER fm-> fm0wer
#define	FM1WER fm-> fm1wer
#define	FM2WER fm-> fm2wer
#define	FM3WER fm-> fm3wer
#define	FMCTRL fm-> fmctrl
#define	FMSTAT fm-> fmstat
#define	FMPSR  fm-> fmpsr
#define	FMSTART fm-> fmstart
#define	FMTRAN  fm-> fmtran
#define	FMPROG  fm-> fmprog
#define	FMPERASE fm-> fmperase
#define	FMMERASE0 fm-> fmmerase0
#define	FMMERASE1 fm-> fmmerase1
#define	FMEND     fm-> fmend
#define	FMMEND    fm-> fmmend
#define	FMRCV     fm-> fmrcv
#define	FMAR0     fm-> fmar0
#define	FMAR1     fm-> fmar1
#define	FMAR2     fm-> fmar2

#define PROGRAM_FLASH_START_ADDR 0x000000 // Starting address of the program flash 
#define PROG_FLASH_START_ADDR 0x00000	// Starting address of the program flash
#define PROG_FLASH_END_ADDR 0x3FFFF     // Ending address of the program flash
#define FLASH_PROGRAM_PAGE_SIZE 1024    // Page Size
#define FLASH_PROGRAM_PAGE_NUM 256			//Number of pages in program flash

/*********************************
 * Flash Data memory Section *
 *********************************/
typedef volatile FARIO struct {
    uint16  fsmibar;
    uint16  fsmibdr;
    uint16  fsm0wer;
    uint16  fsm1wer;
    uint16  fsm2wer;
    uint16  fsm3wer;
	uint16  fsmctrl;
    uint16  fsmstat;
    uint8  fsmpsr;
    uint8  f1;
    uint8  fsmstart;
    uint8  f2;
    uint8  fsmtran;
    uint8  f3;
    uint8  fsmprog;
    uint8  f4;
    uint8  fsmperase;
    uint8  f5;
    uint8  fsmmerase0;
    uint8  f6;
    uint8  fsmmerase1;
    uint8  f7;
    uint8  fsmend;
    uint8  f8;
    uint8  fsmmend;
    uint8  f9;
	uint8  fsmrcv;
    uint8  f10;
	uint16  fsmar0;
	uint16  fsmar1;
	uint16  fsmar2;
} fsm_t;

extern fsm_t * const fsm;
#define FSMIBAR fsm-> fsmibar
#define FSMIBDR fsm-> fsmibdr
#define	FSM0WER fsm-> fsm0wer
#define	FSM1WER fsm-> fsm1wer
#define	FSM2WER fsm-> fsm2wer
#define	FSM3WER fsm-> fsm3wer
#define	FSMCTRL fsm-> fsmctrl
#define	FSMSTAT fsm-> fsmstat
#define	FSMPSR  fsm-> fsmpsr
#define	FSMSTART fsm-> fsmstart
#define	FSMTRAN  fsm-> fsmtran
#define	FSMPROG  fsm-> fsmprog
#define	FSMPERASE fsm-> fsmperase
#define	FSMMERASE0 fsm-> fsmmerase0
#define	FSMMERASE1 fsm-> fsmmerase1
#define	FSMEND     fsm-> fsmend
#define	FSMMEND    fsm-> fsmmend
#define	FSMRCV     fsm-> fsmrcv
#define	FSMAR0     fsm-> fsmar0
#define	FSMAR1     fsm-> fsmar1
#define	FSMAR2     fsm-> fsmar2

/* Bitwise definition for flash data/program memory registers. for both data and prog*/
#define FMBUSY 0x0004
#define FMFULL 0x0008
#define PERR   0x0002
#define EERR   0x0001
#define PER    0x0040
#define PE     0x0020
#define MER    0x0080
#define DERR   0x0010

/* Bitwise definition for flash data/program Protection word */
#define BOOTAREA(p)  (( p & 0x000F ) >> 0)
#define EMPTY(p)     (( p & 0x0070 ) >> 4)
#define ISPE(p)      (( p & 0x0380 ) >> 7)
#define RDPROT(p)    (( p & 0x1C00 ) >> 10)
#define WRPROT(p)    (( p & 0xE000 ) >> 13)

/* Used to determine which FMxER register to use. */
#define FM1WER_START  0x20000

/* 
   Define Data/Prog register setting for 12Mhz, 16Mhz, 20Mhz, 24Mhz for both prog/data, 
   default is for 24Mhz
*/

#define FMPSLR_8Mhz 0x01
#define FMSTART_8Mhz 0x14
#define FMTRAN_8Mhz 0x28
#define FMPROG_8Mhz 0x0E
#define FMPERASE_8Mhz 0x13
#define FMMERASE0_8Mhz 0xC3
#define FMMERASE1_8Mhz 0xC3
#define FMEND_8Mhz 0x14
#define FMMEND_8Mhz 0x32
#define FMRCR_8Mhz 0x04


#define FMPSLR_12Mhz 0x02
#define FMSTART_12Mhz 0x14
#define FMTRAN_12Mhz 0x28
#define FMPROG_12Mhz 0x0E
#define FMPERASE_12Mhz 0x12
#define FMMERASE0_12Mhz 0xC3
#define FMMERASE1_12Mhz 0xC3
#define FMEND_12Mhz 0x14
#define FMMEND_12Mhz 0x32
#define FMRCR_12Mhz 0x04


#define FMPSLR_16Mhz 0x03
#define FMSTART_16Mhz 0x14
#define FMTRAN_16Mhz 0x28
#define FMPROG_16Mhz 0x0E
#define FMPERASE_16Mhz 0x12
#define FMMERASE0_16Mhz 0xC3
#define FMMERASE1_16Mhz 0xC3
#define FMEND_16Mhz 0x14
#define FMMEND_16Mhz 0x32
#define FMRCR_16Mhz 0x04

#define FMPSLR_20Mhz 0x03
#define FMSTART_20Mhz 0x19
#define FMTRAN_20Mhz 0x32
#define FMPROG_20Mhz 0x11
#define FMPERASE_20Mhz 0x18
#define FMMERASE0_20Mhz 0xF4
#define FMMERASE1_20Mhz 0xF4
#define FMEND_20Mhz 0x19
#define FMMEND_20Mhz 0x3E
#define FMRCR_20Mhz 0x05


#define FMPSLR_24Mhz 0x04
#define FMSTART_24Mhz 0x18
#define FMTRAN_24Mhz 0x30
#define FMPROG_24Mhz 0x10
#define FMPERASE_24Mhz 0x17
#define FMMERASE0_24Mhz 0xEA
#define FMMERASE1_24Mhz 0xEA
#define FMEND_24Mhz 0x18
#define FMMEND_24Mhz 0x3C
#define FMRCR_24Mhz 0x05

#define DATA_FLASH_START_ADDR 0x0D0000	// Starting address of the data flash
#define DATA_FLASH_END_ADDR   0x0D1FFF		// Starting address of the data flash
#define FLASH_DATA_PAGE_SIZE  512
#define FLASH_DATA_PAGE_NUM 16			//Number of pages in data flash

/*******************************************************************************************
*
*	ADC Section
*
*******************************************************************************************/
typedef volatile FARIO struct {
	unsigned short adcgcr;
	unsigned short adcacr;
	unsigned short adccntrl;
	unsigned short adcstart;
	unsigned short adcsdly;
	unsigned short adcreslt;
	unsigned short adctest;
	unsigned short adcsmbc0;
	unsigned short adcsmbc1;
	unsigned short adcsmbc2;
	unsigned short adcsmbc3;
	unsigned short adcsmsh;
	}adc_t;

extern adc_t * const adc;

#define ADCGCR adc->adcgcr
#define ADCACR adc->adcacr
#define ADCCNTRL adc->adccntrl
#define ADCSTART adc->adcstart
#define ADCSCDLY adc->adcsdly
#define ADCRESLT adc->adcreslt
#define ADCTEST adc->adctest
#define ADCSMBC0 adc->adcsmbc0
#define ADCSMBC1 adc->adcsmbc1
#define ADCSMBC2 adc->adcsmbc2
#define ADCSMBC3 adc->adcsmbc3
#define ADCSMSH adc->adcsmsh

/* Bit-wise definitions */

/* ADCGCR Register */
#define ADCCLKEN				0x0001
#define ADCIN					0x0002
#define DIFF					0x0004
#define MUX_CFG_0				0x0000
#define MUX_CFG_1				0x0008
#define MUX_CFG_2				0x0010
#define MUX_CFG_3				0x0018
#define MUX_CFG_4				0x0020
#define MUX_CFG_5				0x0028
#define MUX_CFG_6				0x0030
#define MUX_CFG_7				0x0038
#define SAMPLE_X				0x0040
#define SAMPLE_Y				0x0080
#define SAMPLE_Z_PRE_PENDOWN	0x00C0
#define SAMPLE_Z				0x0100
#define PEN_DOWN_DETECT			0x0140
#define PREF_CFG_INTERNAL		0x0000
#define PREF_CFG_VREFP			0x0200
#define PREF_CFG_ADC0			0x0400
#define PREF_CFG_ADC1			0x0600
#define NREF_CFG_INTERNAL		0x0000
#define NREF_CFG_ADC2			0x1000
#define NREF_CFG_ADC3			0x1800
#define ADCINTEN				0x4000
#define MUXOUTEN				0x8000

/* ADCACR */
#define ADC_CLKSEL	0x0001
#define CLK_DIV_0	0x0000
#define CLK_DIV_2	0x0002
#define CLK_DIV_4	0x0004

/* ADCCNTRL */
#define POL		0x0001
#define EXT		0x0002
#define AUTO	0x0004

/* ADCRESLT */
#define RSLT_SIGN		0x1000
#define RSLT_PEN_DOWN	0x2000
#define RSLT_ADC_OFLW	0x4000
#define RSLT_ADC_DONE	0x8000

/****************************************************************
 *
 *  CAN Section
 *
 ****************************************************************/

/* The register structs are split between the Message Buffers
   and the control registers because they are separate
   tables in the spec and there's a good gap suggesting they may 
   wander separately in memory. */

/* CAN Module Message Buffers */
typedef volatile FARIO struct {
  unsigned short cntstat;
  unsigned short tstp;
  unsigned short data3;
  unsigned short data2;
  unsigned short data1;
  unsigned short data0;
  unsigned short id0;
  unsigned short id1;
} can_mbuf_t;

/* Message Buffer Register bit fields */
#define IDE		0x0008		/* ID1 Reg ... */
#define ID1_RTR	0x0010
#define ID28_18	0xFFE0
#define ID17_15	0x0007
#define ID0_RTR	0x0001		/* ID0 Reg ... */
#define ID14_0	0xFFFE
#define ST		0x000F		/* CNTSTAT Reg ... */
#define PRI		0x00F0
#define DLC		0xF000

/* Values for ST in CNTSTAT */
#define MBUF_BUSY			0x01 /* Use just the busy bit for checking rx or tx */
#define ST_RX_NOT_ACTIVE	0x00
#define ST_RX_READY			0x02
#define ST_RX_FULL			0x04
#define ST_RX_OVERRUN		0x06
#define ST_TX_NOT_ACTIVE	0x08
#define ST_TX_RTR			0x0A /* Out of order in spec */
#define ST_TX_ONCE			0x0C
#define ST_TX_ONCE_RTR		0x0E

/* CAN Registers */
typedef volatile FARIO struct {
  unsigned short cgcr;
  unsigned short ctim;
  unsigned short gmskx;
  unsigned short gmskb;
  unsigned short bmskx;
  unsigned short bmskb;
  unsigned short cien;
  unsigned short cipnd;
  unsigned short ciclr;
  unsigned short cicen;
  unsigned short cstpnd;
  unsigned short canec;
  unsigned short cediag;
  unsigned short ctmr;
} can_t;

// This section handles the number of CAN blocks and their register base addrs
#define N_CANBLOCKS 2
#define CAN_MBUF_BASE_ADDRS (can_mbuf_t *) CAN0_MBUF_BASE_ADDR, (can_mbuf_t *) CAN1_MBUF_BASE_ADDR
#define CAN_REG_BASE_ADDRS (can_t *) CAN0_BASE_ADDR, (can_t *) CAN1_BASE_ADDR
#define CAN_IRQS CAN0_IRQ, CAN1_IRQ

/* CAN block uses PE/H6 (CANRX) and PE/H7 (CANTX) */
#define CAN_MUX_PORTS (gpio_t *)PH_BASE_ADDR, (gpio_t *)PE_BASE_ADDR
#define CAN_MUX_MASKS 0xC0, 0xC0

// The control registers
#define CGCR can->cgcr
#define CTIM can->ctim
#define GMSKX can->gmskx
#define GMSKB can->gmskb
#define BMSKX can->bmskx
#define BMSKB can->bmskb
#define CIEN can->cien
#define CIPND can->cipnd
#define CICLR can->ciclr
#define CICEN can->cicen
#define CSTPND can->cstpnd
#define CANEC can->canec
#define CEDIAG can->cediag
#define CTMR can->ctmr

/* Bit field definitions (masks) */

/* CGCR Register */
#define CANEN		0x0001
#define CTX			0x0002
#define CRX			0x0004
#define BUFFLOCK	0x0008 /* Offered through driver API */
#define TSTPEN		0x0010
#define DDIR		0x0020 /* Offered through driver API */
#define LO			0x0040
#define IGNACK		0x0080
#define LOOPBACK	0x0100
#define INTERNAL	0x0200
#define DIAGEN		0x0400
#define EIT			0x0800

/* CTIM register */
#define TSEG2	0x0007
#define TSEG1	0x0078
#define SJW		0x0180
#define PSC		0xFE00

/* CIPND Register */
#define EIPND	0x8000

/* CICLR Register */
#define EICLR	0x8000

/* CSTPND Register */
#define IST		0x000F
#define IRQ		0x0010
#define NS		0x00E0

/* Node Status (NS) field values in CSTPND */
#define NS_NOT_ACTIVE 0x0
#define NS_ERROR_ACTIVE 0x2
#define NS_ERR_WARN_LVL 0x3
#define NS_ERR_PASSIVE 0x4 /* LSb is Don't care */
#define NS_BUS_OFF 0x6 /* LSb is Don't care */

/* CANEC Register */
#define TEC		0x00FF
#define REC		0xFF00

/* CEDIAG Register */
#define EFID	0x000F
#define EBID	0x03F0
#define TXE		0x0400
#define STUFF	0x0800
#define CRC		0x1000
#define MON		0x2000
#define DRIVE	0x4000

/* Error Field Identifier in CEDIAG values */
#define EFID_ERR 0x0
#define EFID_ERR_DEL 0x1
#define EFID_ERR_ECHO 0x2
#define EFID_BUS_IDLE 0x3
#define EFID_ACK 0x4
#define EFID_EOF 0x5
#define EFID_INTERMISSION 0x6
#define EFID_SUSP_TRANS 0x7
#define EFID_SOF 0x8
#define EFID_ARB 0x9
#define EFID_IDE 0xA
#define EFID_EXT_ARB 0xB
#define EFID_R1R0 0xC
#define EFID_DLC 0xD
#define EFID_DATA 0xE
#define EFID_CRC 0xF

/* CTMR Register is all one field - no bit definitions */

#endif /* CP3BT2X_H */
