/************************************************************************
	    $File: //CR/cr16bt/include/drivers.h $
    	$Date: 2003/08/21 $
	    $Revision: #36 $

	Function:	The file contains function the prototypes of the peripheral
	drivers and some common driver definitions.

************************************************************************/

#ifndef DRIVERS_H
#define DRIVERS_H

#include "crcommon.h"
#include "dl.h"

typedef enum {
  RESET_ISP,
  RESET_CLR
} swReset_t;

typedef enum { 
  FLASH_PROGRAM,
  FLASH_DATA,
  INFO_BLOCK,
  RAM,
  CFI
} memory_t;

typedef enum {
  FREQ_8MHz,
  FREQ_12MHz,
  FREQ_16MHz,
  FREQ_20MHz,
  FREQ_24Mhz
} frequency_t;

/* IAR Tools Definitions */
#ifdef __IAR_SYSTEMS_ICC__
#  include <intrinsics.h>
#  define set_i_bit() __set_PSR_I_bit()
#  define _di_() __disable_interrupt()
#  define _ei_() __enable_interrupt()
#  define _nop_() __no_operation()
#  define INTERRUPT __interrupt
#  define TRAP __trap
#  define FARIO __data24
#  define FAR __data24
#endif

/* National Tools Definitions */
#ifdef __NSC_CRCC__	/* National Tools */
#  include <asm.h>
#  define __set_PSR_I_bit() set_i_bit()
#  define __disable_interrupt() _di_()
#  define __enable_interrupt() _ei_()
#  define __no_operation() _nop_()
#  define INTERRUPT
#  define TRAP
#  define FARIO __far
#  define FAR __far
#endif


/***********************************************************************
 *
 * Traps section
 *
 ***********************************************************************/
/* For uc/os-ii the SVC ISR should be the OS context switch function */
#define SVC_ISR OSCtxSw

#ifdef __NSC_CRCC__
extern TRAP void null_isr(void);
extern TRAP void und_isr(void);
extern TRAP void iad_isr(void);
extern TRAP void SVC_ISR(void);
#endif

/***********************************************************************
 *
 * GPIO section
 *
 ***********************************************************************/

typedef enum
{
	PORTA = 0,
	PORTB = 1,
	PORTC = 2,
	PORTD = 3,
	PORTE = 4,
	PORTF = 5,
	PORTG = 6,
	PORTH = 7,
	PORTI = 8,
	PORTJ = 9,
	PORTK = 10,
	PORTL = 11
} port_t;

typedef enum
{
	GPIO_INPUT,
	GPIO_OUTPUT
} gpio_dir_t; 

typedef enum
{
	GPIO_ALT_SRC_A,
	GPIO_ALT_SRC_B
}gpio_alt_src_t;


extern void gpio_set_alt(port_t port, unsigned char mask, boolean mode);
extern void gpio_set_alt_src(port_t port, unsigned char mask,
								gpio_alt_src_t source);
extern void gpio_set_dir(port_t port, unsigned char mask, gpio_dir_t dir);
extern void gpio_write(port_t port, unsigned char mask, unsigned char data);
extern unsigned char gpio_read(port_t port, unsigned char mask);


/***********************************************************************
 *
 * ICU section
 *
 ***********************************************************************/

extern void icu_init(void);
extern void icu_mask(unsigned int);
extern void icu_unmask(unsigned int);

/***********************************************************************
 *
 * System Configuration	section
 *
 ***********************************************************************/

extern void SYSCFGEnableIoExpansion(void);


/***********************************************************************
 *
 * BIU section
 *
 ***********************************************************************/

#define ZONE0 0
#define ZONE1 1
#define ZONE2 2
#define IOZONE 0xff

extern void biu_config_zone(unsigned int, unsigned int, unsigned int, boolean);
extern void biu_set_early_write(boolean);


/***********************************************************************
 *
 * USART section
 *
 ***********************************************************************/

typedef enum {BPS_9600 = 0, BPS_38400, BPS_115200, BPS_230400, BPS_460800, BPS_921600} bitrate_t;
#define NUM_BITRATE    (BPS_921600 + 1)
typedef enum {NO_FLOW_CONTROL, HW_FLOW_CONTROL, SW_FLOW_CONTROL} flow_control_t;

extern void usart_init(unsigned int, bitrate_t, flow_control_t, void (*)(void));
extern void usart_shutdown(unsigned int);
extern unsigned int usart_tx(unsigned int, unsigned char *, unsigned int);
extern unsigned int usart_rx(unsigned int, unsigned char *, unsigned int);
extern unsigned int usart_rx_count(unsigned int);

extern INTERRUPT void usart0_tx_isr(void);
extern INTERRUPT void usart0_rx_isr(void);
extern INTERRUPT void usart1_tx_isr(void);
extern INTERRUPT void usart1_rx_isr(void);
extern INTERRUPT void usart2_tx_isr(void);
extern INTERRUPT void usart2_rx_isr(void);
extern INTERRUPT void usart3_tx_isr(void);
extern INTERRUPT void usart3_rx_isr(void);

/***********************************************************************
 *
 * TWM section
 *
 ***********************************************************************/

extern volatile unsigned int tick_flag;
extern void twm_init(unsigned int, void (*)(void));
extern void wd_init(unsigned char);
extern void wd_restart(void);
extern INTERRUPT void twm_isr(void);

/***********************************************************************
 *
 * Triple Clock and Reset module section
 *
 ***********************************************************************/

/* The following is an enumerated type for clock frequencies. Each value
   corresponds to one of the clock frequencies supported by the CP3BT2X/CP3BT1X
*/
typedef enum {_6MHZ=0, _12MHZ, _18MHZ, _24MHZ, _30MHZ, _36MHZ, _12MHZPLL, _16MHZPLL } freq_t;  

extern void TCRSetClock(freq_t);
extern void TCRPllEnable(void);


/***********************************************************************
 *
 * MFT section
 *
 ***********************************************************************/

//extern void mft_init(const int ClockTick);
extern void mft_init(const int mode, const int ClockTick1, const int ClockTick2, const int ClockTick3);
extern INTERRUPT void mfta_isr(void);
extern INTERRUPT void mftb_isr(void);

//extern void  InitMFT(uint8, uint16, uint16, uint16);
//extern void  MFTDelay(uint16);
extern void mft_delay(const int mode, const int Channel, unsigned short Dly);

/***********************************************************************
 *
 * VTU section
 *
 ***********************************************************************/

extern void vtu_init(const int Channel, const int VTUmode, const int Count, 
                     const int Duty, const int Period, const int Prescaler);
extern INTERRUPT void vtua_isr(void);
extern INTERRUPT void vtub_isr(void);
extern INTERRUPT void vtuc_isr(void);
extern INTERRUPT void vtud_isr(void);

/***********************************************************************
 *
 * Microwire/SPI section
 *
 ***********************************************************************/

extern void MWInit(void);

extern void mw_init(boolean, unsigned int, void (*)(void));
extern void mw_tx(unsigned char *, unsigned int);
extern void mw_rx(unsigned char *, unsigned int);
extern void mw_tx_rx(unsigned char *, unsigned char *, unsigned int);
extern INTERRUPT void mw_isr(void);

/***********************************************************************
 *
 * Access.bus section
 *
 ***********************************************************************/

extern void acbs_init(unsigned char address);
extern unsigned int acbs_tx(unsigned char *data, unsigned short size);
extern unsigned int acbs_rx(unsigned char *inbuf, unsigned short size);
extern unsigned int acbs_rx_count(void);
extern void acbm_init(unsigned int, unsigned short divisor, void (*callback_p)(void));
extern void acbm_tx(unsigned int, unsigned char address, unsigned char *data, unsigned short size,
							boolean stop_flag);
extern void acbm_rx(unsigned int, unsigned char address, unsigned char *inbuf, unsigned short size,
							boolean stop_flag);
extern INTERRUPT void acb0_isr(void);
extern INTERRUPT void acb1_isr(void);


/***********************************************************************
 *
 * Random Number Generator (RNG) section
 *
 ***********************************************************************/
extern void rng_init( uint32 clk_ratio );
extern uint rng_get( void rng_cbk( uint rand ) );
extern void rng_shutdown( void );
extern INTERRUPT void rng_isr();


/***********************************************************************
 *
 * DMA section
 *
 ***********************************************************************/
extern void usart_dma_init(bitrate_t , flow_control_t,
                           void (*)(void));
extern unsigned int usart_dma_tx(unsigned char *, unsigned int);
extern unsigned int usart_dma_rx(unsigned char *, unsigned int);
extern unsigned int usart_dma_rx_count(void);
extern INTERRUPT void dma_channel0_isr(void);
extern INTERRUPT void dma_channel1_isr(void);
extern INTERRUPT void dma_channel2_isr(void);
extern INTERRUPT void dma_channel3_isr(void);


/***********************************************************************
 *
 * USB section
 *
 ***********************************************************************/

#define N_ENDPOINTS 7

#define ep_from_rxep(rxep)	((rxep) << 1) // Which endpoint is this RX endpoint
#define ep_from_txep(txep)	(((txep)<< 1) - 1) // Which endpoint is this TX endpoint
#define txep_from_ep(ep) (((ep) + 1) >> 1)
#define rxep_from_ep(ep) ((ep) >> 1)

// Used to define which EP is being used by user
// DJC - check no t/rxep0 conflicts here
typedef enum { RXTXEP0=0, TXEP1=1, RXEP1=2, 
		TXEP2=3, RXEP2=4, TXEP3=5, RXEP3=6 } ep_id_t;

/* Defines Standard Device Requests for EP0 (Table 9-4) */
#define GET_STATUS              0x00  
#define CLEAR_FEATURE           0x01
// 0x02 is reserved
#define SET_FEATURE             0x03
// 0x04 is reserved
#define SET_ADDRESS             0x05 
#define GET_DESCRIPTOR          0x06
#define SET_DESCRIPTOR          0x07 
#define GET_CONFIGURATION       0x08 
#define SET_CONFIGURATION       0x09 
#define GET_INTERFACE           0x0A
#define SET_INTERFACE           0x0B
#define SYNCH_FRAME             0x0C

/* Defines Descriptor Types (Table 9-5) */
#define DEVICE						0x01
#define CONFIGURATION				0x02
#define STRING						0x03
#define INTERFACE					0x04
#define ENDPOINT					0x05
#define DEVICE_QUALIFIER			0x06
#define OTHER_SPEED_CONFIGURATION	0x07
#define INTERFACE_POWER				0x08

/* Defines Standard Feature Selectors (Table 9-6) */
#define DEVICE_REMOTE_WAKEUP		0x01
#define ENDPOINT_HALT				0x00
#define	TEST_MODE					0x02

/* Defines bitmap of bmRequestType */
#define DATADIRECTION				0x80
#define TYPE						0x60
#define	RECIPIENT					0x1F

/* Defines subparts of bmRequestType */
#define TYPESTANDARD				0x00
#define TYPECLASS					0x20
#define	TYPEVENDOR					0x40
#define RECIPDEVICE					0x00
#define RECIPINTERFACE				0x01
#define RECIPENDPOINT				0x02
#define	RECIPOTHER					0x03

// Defines for supporting resume functions
#define SIGNAL_RESUME				0x01
#define GO_OPERATIONAL				0x02


/* Macro for clearing structures to zero */
#define BZERO(buf) memset(&(buf), 0, sizeof(buf))



//-----------------------------------------------------------------
//                       DESCRIPTORS STRUCTURES
//-----------------------------------------------------------------

#ifdef __IAR_SYSTEMS_ICC__
#pragma pack(push)
#pragma pack(1)
#endif

// Standard Device Descriptor as per USB 2.0 Sec. 9.6.1 Table 9-8
typedef struct  {
  uint8  bLength;				// Size of descriptor in bytes
  uint8  bDescriptorType;		// DEVICE descriptor type (constant)
  uint16 bcdUSB;				// BCD code USB Spec Release Number (i.e 1.10 is 0110h)
  uint8  bDeviceClass;			// Class code as defined by USB-IF
  uint8  bDeviceSubClass;		// Sub Class code as defined by USB-IF
  uint8  bDeviceProtocol;		// Protocol Code as defined by USB-IF
  uint8  bMaxPacketSize0;		// Max packet size for EP0
  uint16 idVendor;				// Vendor ID as assigned by USB-IF
  uint16 idProduct;				// Product ID as assigned by developer
  uint16 bcdDevice;				// BCD release number (e.g. 1.0.9 is 0109h)
  uint8  iManufacturer;			// Index to manufacturers name string (UNICODE)
  uint8  iProduct;				// Index to product description string (UNICODE)
  uint8  iSerialNumber;			// Index to product serial number as a string (UNICODE)
  uint8  bNumConfigurations;	// Number of possible configurations
} UsbDeviceDescriptorType;

// Configuration
typedef struct {
  uint8            bLength;
  uint8            bDescriptorType;
  uint16           wTotalLength;
  uint8            bNumInterfaces;
  uint8            bConfigurationValue;
  uint8            iConfiguration;
  uint8            bmAttributes;
  uint8            maxPower;
} UsbConfigurationDescriptorType;

//--------------------     Interface    ----------------------------
typedef struct {
  uint8 bLength;
  uint8 bDescriptorType;
  uint8 bInterfaceNumber;
  uint8 bAlternateSetting;
  uint8 bNumEndpoints;
  uint8 bInterfaceClass;
  uint8 bInterfaceSubClass;
  uint8 bInterfaceProtocol;
  uint8 iInterface;
} UsbInterfaceDescriptorType;

// Endpoint
typedef struct {
  uint8  bLength;
  uint8  bDescriptorType;
  uint8  bEndpointAddress; // Decode using mask defined above
  uint8  bmAttributes;
  uint16 wMaxPacketSize;
  uint8  bInterval;
} UsbEndpointDescriptorType;

// Special structure to support concatanation of descriptors
// for multiple configurations
typedef struct
{
	uint8 * ucpConfigDesc;
	uint16 usLengthOfDesc;
} t_TotalDescriptorType;

#ifdef __IAR_SYSTEMS_ICC__
#pragma pack(pop)
#endif

//-----------------------------------------------------------------
//                      END OF DESCRIPTORS STRUCTURES
//-----------------------------------------------------------------

// Special structure for setting up active endpoints and iso ep's
// for different Configs, Interfaces and Alt Settings
typedef struct 
{
	uint8 bActiveEndpoint;
	uint8 bIsoEndpoint;
} t_EndpointSetup;

/* Union for holding Standard Device Request data */
typedef union
{
	uint8 byte[8];
	struct
	{
		uint8   bmRequestType;
		uint8   bRequest;
		uint16  wValue;
		uint16  wIndex;
		uint16  wLength;
	} field;
} usb_request_t;

typedef enum { USB_CONFIGURED, USB_SD3_SUSPEND, USB_SD5_SUSPEND, 
	USB_RESUME, USB_EP_EVT, USB_WARN_EVT, USB_FRAME_EVT } usb_evt_t;

// Callbacks to indicate certain events to user
typedef void (*evt_clbk_t) ( usb_evt_t evt, ep_id_t ep );
typedef void (*vendor_clbk_t)( usb_request_t USBRequest );
typedef void (*class_clbk_t) ( usb_request_t USBRequest );
typedef void (*set_desc_clbk_t) ( usb_request_t USBRequest );

typedef struct
{
	evt_clbk_t			evt_clbk;
	vendor_clbk_t		vendor_clbk;
	class_clbk_t		class_clbk;
	set_desc_clbk_t		set_desc_clbk;
} usb_init_info_t;

typedef enum { BUFFEREMPTY, TRNSFRING, TRNSFRCOMPLETE, TRNSFRSTOPPED }
	ep_xfer_stat_t;

typedef struct
{
	ep_xfer_stat_t 	ucTransferStatus;
	uint xfer_cnt;
} usb_ep_stat_t;

/* Defines current device state as per figure 9-1 USB2.0 */
typedef enum { NOT_ATTACHED, ATTACHED, POWERED, DEFAULT, ADDRESS, CONFIGURED } 
	usb_dev_state_t;

/* Structure to allow us to give device information to user */
typedef struct
{
	uint8 ucUSBAddress;
	uint8 ucUSBConfiguration;
	uint8 ucUSBInterface;
	uint8 ucUSBAlternate;
	usb_dev_state_t ucDeviceState;
	uint8 ucWakeUpEnable;			// Holds whether device currently has Wake Up enabled
	uint8 ucSelfPowered;			// Holds whether we are currently self powered
	boolean rem_wake_ok;
	boolean suspended;
} usb_node_stat_t;

// The public API
// From usb.c
extern boolean usb_init( usb_init_info_t *usbinitinfo );
extern boolean node_resume (uint8);
extern usb_node_stat_t get_node_status (void);
extern void usb_shutdown( void );
// From usbhw.c
extern void get_ep_stat( ep_id_t ep, ep_xfer_stat_t *curr_state, uint *xfer_cnt );
extern void stall_ep0();
extern void stall_ep(ep_id_t ep);
extern void clear_ep_stall(ep_id_t ep); 
extern uint usb_tx_data (ep_id_t ep, uint8 *data_addr, uint len );
extern uint usb_rx_data (ep_id_t ep, uint8 *data_addr, uint len );
extern void sendEP0data (uint req_len, uint avail_len, uint8 *data_addr);

/**********************************************************************
 * Advanced Audio Interface
 * 
 ***********************************************************************/

#define CODEC_MUTE  0x01

typedef enum {NOCVSDCONV = 3, CVSD2LINEARPCM = 2, CVSD2ULAW = 0, CVSD2ALAW = 1} audio_conv_t;

typedef struct {
	boolean async_mode;
	unsigned char sample_length;
	boolean long_fs;
	unsigned char frame_sync_length;
	boolean invert_fs;
	unsigned char num_of_slots;
	unsigned char bitclock_prescalar;
	unsigned char bits_per_frame;
	unsigned char assigned_tx_slots;
	unsigned char assigned_rx_slots;
	audio_conv_t audio_conv;
}audio_config_t;

extern void audio_init(audio_config_t *, dl_t *, dl_t *, void (*)(unsigned int));
void audio_set_control_bits(unsigned int, unsigned char);
extern void audio_start(void);
extern void audio_stop(void);
extern INTERRUPT void aai_isr(void);
extern INTERRUPT void pcm2cvsd_isr(void);

extern uint16   AAIInit (uint16, uint16, uint16, uint16);
extern INTERRUPT void aai_isr (void);


/**********************************************************************
 * CVSD registers Section
 * 
 ***********************************************************************/
extern uint16   CVSDInit (uint16 Config);
extern INTERRUPT void pcm2cvsd_isr (void);

/***********************************************************************
 *
 * Bluetooth LLC section
 *
 ***********************************************************************/

extern void btllc_init(void);
extern INTERRUPT void bt_llc0_isr(void);


/**********************************************************************
*
*	TIMER Section
*
***********************************************************************/
extern void timer_init(void);
extern int timer_set_wakeup(void (* func)(void), unsigned int, boolean);
extern void timer_clear_wakeup(int );
extern void timer_wakeup_check(void);
extern boolean timer_wait(unsigned int);


/****************************************************************
 *
 * Flash Programming Section
 *
 ****************************************************************/
typedef enum {
  FLASH_PROG_ERR,
  FLASH_ERASE_ERR
} flash_status_t;

extern void FlashRead(memory_t Type, 
				unsigned long Offset,
				unsigned short Size,
				unsigned char *Data );

extern void FlashInit(memory_t Type, frequency_t Frequency);
extern unsigned short FlashInformationBlockRead(memory_t Type, 
					 unsigned short Offset, 
					 unsigned short *Data, 
					 unsigned short Size  );

extern unsigned short FlashGetPageSize(memory_t Type);
extern unsigned short FlashGetPageNum(memory_t Type);
extern unsigned long FlashGetStartAddress(memory_t Type);
           
#ifdef ISP_EXISTS          
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"
#endif           
#endif
extern unsigned short FlashInformationBlockWrite(memory_t Type,
					  unsigned short Offset, 
					  unsigned short *Data, 
					  unsigned short Size);
#ifdef ISP_EXISTS            
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"           
#endif
#endif
extern void FlashInformationBlockErase(memory_t Type, 
				unsigned short Offset, 
				unsigned char Data );
#ifdef ISP_EXISTS      
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"           
#endif
#endif
extern void FlashPageProgramSafe (memory_t Type, 
		  unsigned long Offset, 
		  unsigned short Size,  
		  unsigned char *InBuf);

#ifdef ISP_EXISTS
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"           
#endif
#endif
extern void FlashPageProgram(memory_t Type, 
		  unsigned long Offset, 
		  unsigned short Size, 
		  unsigned char *RefBuf, 
		  unsigned char *InBuf);
      
#ifdef ISP_EXISTS
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"
#endif           
#endif
extern void FlashEraseAll(memory_t Type );

#ifdef ISP_EXISTS
#ifdef __IAR_SYSTEMS_ICC__
#pragma location="FLASH_MOD_SEG"           
#endif
#endif
extern void FlashPageErase(memory_t Type, unsigned long Address);


/****************************************************************
 *
 * ADC Section
 *
 ****************************************************************/

extern void AdcInit(boolean Diff, unsigned char Channel, unsigned short PRef,unsigned short NRef,
	boolean ExternalWakeup, boolean TouchScreen,boolean Repeat,void (*adc_result_callback_p)(volatile short *Result));
extern void AdcTouchScreenConfig(unsigned short CoOrdinate);
extern void AdcStartConversion(void);
extern short AdcReadResult(void);

extern INTERRUPT void adc_isr(void);


boolean IsSwResetValid(void);
void SwReset(swReset_t func);


/****************************************************************
 *
 * CAN Section
 *
 ****************************************************************/

#define CAN_NUM_MBUFS 15

typedef enum { CAN_DEFAULTS = 0x0000, // Buffs unlocked, default data direction
               CAN_BUFFLOCK = 0x0008, // Lock, full buffers not overwritten
               CAN_DDIR = 0x0020 // Reverse data storage, data1 at lowest addr
} can_config_t;

typedef enum { CAN_STD, // Standard frame
               CAN_EXT  // Extended frame
} msg_type_t;

typedef enum { SEND_DATA = 0x0C,          // Send Data Frame
	           REMOTE_REQ = 0x00,         // Send Remote frame
			   RESPOND_TO_REMOTE = 0x0A,  // Delay send until remote received
	           SEND_THEN_RTR = 0x0E,      // Send then enter RTR mode
			   TX_CANCEL = 0x08           // Cancel current sending/RTR mode
} can_tx_type_t;

typedef enum { CANERR_BADFREQ, ERR_MBUF_BUSY } can_err_t; 

void can_init( 
		unsigned int can_id,
		can_config_t global_config,
		unsigned int  bus_freq_ratio, 
		unsigned short global_mask, 
		unsigned short global_mask_ext, 
		unsigned short basic_mask, 
		unsigned short basic_mask_ext, 
		void   (*mbuf_callback)(unsigned short mbuf_num, 
								unsigned short cediag ) );

unsigned int can_config_tx( 
		unsigned int  can_id,
		unsigned char mbuf_num, 
		unsigned long id, 
		unsigned char *msg, 
		unsigned int  len,
		msg_type_t    msg_type,
		unsigned int  priority,
		can_tx_type_t tx_type );

unsigned int can_config_rx( 
		unsigned int  can_id,
		unsigned char mbuf_num, 
		unsigned long id, 
		msg_type_t    msg_type );

unsigned int can_read_msg( 
		unsigned int   can_id,
		unsigned char  mbuf_num, 
		unsigned long *id, 
		unsigned char *msg,
		msg_type_t    *msg_type,
		unsigned int  *priority,
		unsigned short *tstamp );

void can_shutdown(unsigned int can_id);


extern INTERRUPT void can0_isr(void);
extern INTERRUPT void can1_isr(void);

/****************************************************************
 *
 * Serial Interface Bit Banging Section
 *
 ****************************************************************/

/* This structure describes an SPI-like serial interface. */
typedef struct {
	port_t port;			/* A GPIO that includes the interface pins. */
	unsigned char clkmsk;	/* The clock pin mask. */
	unsigned char datamsk;	/* The data pin mask. */
	unsigned char enmsk;	/* The enable pin mask. */
} pindesc_t;

extern void sibb_write(pindesc_t *pindesc, unsigned char *data_p,
						unsigned int size);

#endif /* DRIVERS_H */
