/************************************************************************
	File Name : dbgmsg.c

	Function:	This file contain some utility functions for debug
	messages.

************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "drivers.h"

static char dbg_msg_buf[80];
static unsigned int (* txfunc)(unsigned char *, unsigned int );

/******************************************************************************  
    Function: dbg_init

	Description
	-----------
    This function initializes the the dbug message utility module.

	Arguments
	---------
	tx_func:
	A pointer to a function that will transmit debug messages.
*******************************************************************************/
void dbg_init(unsigned int (* tx_func)(unsigned char *, unsigned int ))
{
	txfunc = tx_func;

}

/******************************************************************************  
    Function: dbg_msg

	Description
	-----------
    This function prints the debug message. This function receives a variable
	number of arguments following the printf convention.

	Arguments
	---------
	format:
	A printf style format string.

    ...
	Additional argument as required by the format.
*******************************************************************************/
void dbg_msg(char * format, ...)
{
	int msg_size;
	va_list ap;

	va_start(ap, format);

	msg_size = vsprintf(dbg_msg_buf, format, ap);
	if (dbg_msg_buf[msg_size-1] == '\n')
		dbg_msg_buf[msg_size++] = '\r'; //Add \r for hyper-terminal

	(*txfunc)((unsigned char *)dbg_msg_buf, (unsigned int)msg_size);

}

/******************************************************************************  
    Function: dbg_print_bdaddr

	Description
	-----------
    This function formats and prints a BD address.

	Arguments
	---------
	bd_addr:
	A pointer to a 6-byte array containing the BD address.

    ...
	Additional argument as required by the format.
*******************************************************************************/
void dbg_print_bdaddr(unsigned char *bd_addr)
{
	sprintf(dbg_msg_buf, "%02x %02x %02x %02x %02x %02x ",
		bd_addr[5], bd_addr[4], bd_addr[3], bd_addr[2], bd_addr[1], bd_addr[0]);
	(*txfunc)((unsigned char *)dbg_msg_buf, 18);
}
