/************************************************************************
*	File Name:	sbrk.c
*
*	Function:	The following 'sbrk' function is used to override the default 
*				implementation in the National Tools C library. This is due to
*				the current implementation which prevents us for using it in a
*				multi-task/multi-stack environment. In the future this problem
*				is expected to be fixed and then this function can be removed.
************************************************************************/
#include <stdlib.h>
#include <errno.h>
#include "crcommon.h"

extern char heap_buffer[];
extern char * end_of_heap;

static char * heap_ptr;				/* pointer to head of heap */
static boolean first_time = TRUE;	/* Indicates first call or not. */



/******************************************************************************  
    Function: sbrk

	Description
	-----------
    This function extends heap space.

	Arguments
	---------
	size:
	The number of additional bytes required for the heap.

	Return Value
	------------
	Start of new space allocated, or -1 for errors.
*******************************************************************************/
char * sbrk(int size)
{
    char *	old_heap_ptr;
    volatile char *	s;

    /* heap_ptr is initialized to the heap buffer start */
    if (first_time) {
		heap_ptr = (char *) heap_buffer;
		first_time = FALSE;
    }

    old_heap_ptr = heap_ptr;
    s = heap_ptr + size;    /* top of heap iff all goes well */

    if (s > (char *) end_of_heap) {
		errno = ENOMEM;
  		return ((char *) -1);
    }

    /* success: update heap_ptr and return previous value */
    heap_ptr += size;
    return (old_heap_ptr);
}

