/*********************************************************************************************
	
	File Name:		vt100.c

	Description:	This file contains the implementation of the function for the vt100 
					utilities, e.g., Setting the cursor position, setting the text color, etc.
	
	!--Note:		The actual character strings, defined here, are in the same order as they 
					appear in the vt100.h header file.

**********************************************************************************************/

#include "vt100.h"
#include "drivers.h"
#include <stdarg.h>

const char escSeq[][10] = 
{
	"\033[%d;%df",		/* Set Cursor position	*/
	"\033[s",			/* Save Cursor position */
	"\033[u",			/* Restore Cursor position */
	"\033[%d;%dH",		/* Set Cursor Home */
	"\033[%dD",			/* Move Cursor Backward */
	"\033[%dC",			/* Move Cursor Forward */
	"\033[2K",			/* Erase Line */
	"\033[2J",			/* Erase the whole screen */
	"\033[7h",			/* Enable Line Wrap */
	"\033[%d;%dr",		/* Enable Scroll Screen */
	"\033[r",			/* Disable Scroll Screen */
	"\033[0m",			/* Disable All Attributes */
	"\033[1m",			/* Set text bright */
	"\033[21m",			/* Reset text bright */
	"\033[2m",			/* Set text Dull */
	"\033[22m",			/* Reset text dull */
	"\033[3m",			/* Set text blinking */
	"\033[23m",			/* Reset text blinking */
	"\033[4m",			/* Set text underscore */
	"\033[24m",			/* Reset text underscore */
	"\033[31m",			/* Set text Red */
	"\033[32m",			/* Set text Green */
	"\033[34m",			/* Set text Blue */
	"\033[41m",			/* Set Background Red */
	"\033[42m",			/* Set background Green */
	"\033[44m"			/* Set background Blue */
};

void VT100Action(int CommandCode, ...)
{
	char Buffer[10];
	int length, Row, Col;
	va_list aPtr;
	va_start (aPtr, CommandCode);
	switch (CommandCode)
	{
		//Process addtional Arguments for the calls, which require them
	case SETCURSORPOSITION: case SETCURSORHOME: case ENABLESCROLLSCREEN:	//For SETCURSORPOSITION, SETCURSORHOME, ENABLESCROLLSCREEN
		
		Row = va_arg(aPtr, int);
		Col = va_arg(aPtr, int);
		break;
    
	case SETCURSORBACKWARDS: case SETCURSORFORWARD:		//For SETCURSORBACKWARDS, SETCURSORFORWARD
		  Row = va_arg(aPtr, int);
		
		  break;
	
  default:
		break;
	}

	length = sprintf(Buffer, &escSeq[CommandCode][0], Row, Col);

	if((usart_dma_tx((unsigned char *) &Buffer,length)) != length)
		panic(); //the USART can't transmit

  	va_end(aPtr);
}
